@@ -17,12 +17,13 @@
 #include <gdk_imlib.h>
 
 #include "log.h"
+#include "mem.h"
 
 #include "gui_main.h"
 
 #include "colors.h"
 
-struct rgbtriple {
+static struct rgbtriple {
   int r, g, b;
 } colors_standard_rgb[COLOR_STD_LAST] = {
   {  0,   0,   0},  /* Black */
@@ -60,7 +61,7 @@ static void alloc_standard_colors (void)
   int i;
 
   for (i=0, cmap=gtk_widget_get_default_colormap (); i<COLOR_STD_LAST; i++) {
-    colors_standard[i]       = g_malloc(sizeof(GdkColor));
+    colors_standard[i]       = fc_malloc(sizeof(GdkColor));
 
     colors_standard[i]->red  = colors_standard_rgb[i].r<<8;
     colors_standard[i]->green= colors_standard_rgb[i].g<<8;
@@ -108,3 +109,15 @@ void init_color_system(void)
 {
   alloc_standard_colors();
 }
+
+/*************************************************************
+...
+*************************************************************/
+void free_color_system(void)
+{
+  int i;
+
+  for (i = 0; i < COLOR_STD_LAST; i++) {
+    free(colors_standard[i]);
+  }
+}
