@@ -54,25 +54,6 @@
 #include "mapclass.h"
 #include "overviewclass.h"
 
-/*
-The bottom row of the map was sometimes hidden.
-
-As of now the top left corner is always aligned with the tiles. This is what
-causes the problem in the first place. The ideal solution would be to align the
-window with the bottom left tiles if you tried to center the window on a tile
-closer than (screen_tiles_height/2 -1) to the south pole.
-
-But, for now, I just grepped for occurences where the ysize (or the values
-derived from it) were used, and those places that had relevance to drawing the
-map, and I added 1 (using the EXTRA_BOTTOM_ROW constant).
-
--Thue
-*/
-
-/* If we have isometric view we need to be able to scroll a little extra down.
-   The places that needs to be adjusted are the same as above. */
-#define EXTRA_BOTTOM_ROW (is_isometric ? 6 : 1)
-
 /**************************************************************************
  Some support functions
 **************************************************************************/
@@ -97,7 +78,8 @@ int get_map_y_visible(void)
 }
 
 /**************************************************************************
- ...
+ This function is called to decrease a unit's HP smoothly in battle
+ when combat_animation is turned on.
 **************************************************************************/
 void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
 			     struct unit *punit1, int hp1)
@@ -105,16 +87,6 @@ void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
   static struct timer *anim_timer = NULL; 
   struct unit *losing_unit = (hp0 == 0 ? punit0 : punit1);
 
-  if (!do_combat_animation) {
-    punit0->hp = hp0;
-    punit1->hp = hp1;
-
-    refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
-    refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
-
-    return;
-  }
-
   set_units_in_combat(punit0, punit1);
 
   do
@@ -129,8 +101,8 @@ void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
     else
       punit0->hp--;
 
-    refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
-    refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
+    refresh_tile_mapcanvas(punit0->x, punit0->y, TRUE);
+    refresh_tile_mapcanvas(punit1->x, punit1->y, TRUE);
 
     usleep_since_timer_start(anim_timer, 10000);
 
@@ -140,8 +112,8 @@ void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
 
   set_units_in_combat(NULL, NULL);
 
-  refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
-  refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
+  refresh_tile_mapcanvas(punit0->x, punit0->y, TRUE);
+  refresh_tile_mapcanvas(punit1->x, punit1->y, TRUE);
 }
 
 /**************************************************************************
@@ -180,17 +152,20 @@ void set_overview_dimensions(int x, int y)
 /**************************************************************************
 ...
 **************************************************************************/
-void update_turn_done_button(int do_restore)
+void update_turn_done_button(bool do_restore)
 {
-  static int flip;
+  static bool flip = FALSE;
 
-  if (game.player_ptr->ai.control && !ai_manual_turn_done)
+  if (!get_turn_done_button_state()) {
     return;
+  }
 
-  if ((do_restore && flip) || !do_restore)
-  {
-    if (!flip) set(main_turndone_button,MUIA_Background, MUII_FILL);
-    else set(main_turndone_button,MUIA_Background, MUII_ButtonBack);
+  if ((do_restore && flip) || !do_restore) {
+    if (!flip) {
+      set(main_turndone_button, MUIA_Background, MUII_FILL);
+    } else {
+      set(main_turndone_button, MUIA_Background, MUII_ButtonBack);
+    }
     flip = !flip;
   }
 }
@@ -216,7 +191,8 @@ void update_info_label(void)
 {
   int d;
 
-  settextf(main_people_text, _("Population: %s"), int_to_text(civ_population(game.player_ptr)));
+  settextf(main_people_text, _("Population: %s"),
+	   population_to_text(civ_population(game.player_ptr)));
   settextf(main_year_text, _("Year: %s"), textyear(game.year));
   settextf(main_gold_text, _("Gold: %d"), game.player_ptr->economic.gold);
   settextf(main_tax_text, _("Tax:%d Lux:%d Sci:%d"),
@@ -270,7 +246,8 @@ void update_unit_info_label(struct unit *punit)
     int units = 0;
     pcity = player_find_city_by_id(game.player_ptr, punit->homecity);
 
-    settextf(main_unitname_text, "%s%s", get_unit_type(punit->type)->name, (punit->veteran) ? _(" (veteran)") : "");
+    settextf(main_unitname_text, "%s%s", unit_type(punit)->name,
+	     (punit->veteran) ? _(" (veteran)") : "");
     settext(main_moves_text, (hover_unit == punit->id) ? _("Select destination") : unit_activity_text(punit));
     settext(main_terrain_text, map_get_tile_info_text(punit->x, punit->y));
     settext(main_hometown_text, pcity ? pcity->name : "");
@@ -370,142 +347,41 @@ void set_indicator_icons(int bulb, int sol, int flake, int gov)
 }
 
 /**************************************************************************
- GUI Independ (with new access functions)
-**************************************************************************/
-void refresh_tile_mapcanvas(int x, int y, int write_to_screen)
-{
-  x = map_adjust_x(x);
-  y = map_adjust_y(y);
+Finds the pixel coordinates of a tile.  Beside setting the results in
+canvas_x,canvas_y it returns whether the tile is inside the visible
+map.
 
-  if (tile_visible_mapcanvas(x, y))
-  {
-    update_map_canvas(x,y, 1, 1, write_to_screen);
-  }
-  overview_update_tile(x, y);
-}
-
-
-/**************************************************************************
-Finds the pixel coordinates of a tile.
-Beside setting the results in canvas_x,canvas_y it returns whether the tile
-is inside the visible map.
+This function is almost identical between all GUI's.
 **************************************************************************/
 int get_canvas_xy(int map_x, int map_y, int *canvas_x, int *canvas_y)
 {
-  int map_canvas_store_twidth = xget(main_map_area, MUIA_Map_HorizVisible);
   int map_view_x0 = xget(main_map_area, MUIA_Map_HorizFirst);
   int map_view_y0 = xget(main_map_area, MUIA_Map_VertFirst);
+  int width = _mwidth(main_map_area);	/* !! */
+  int height = _mheight(main_map_area);	/* !! */
 
-  if (is_isometric) {
-    /* canvas_x,canvas_y is the top corner of the actual tile, not the pixmap.
-       This function also handels non-adjusted tile coords (like -1, -2) as if
-       they were adjusted.
-       You might want to first take a look at the simpler city_get_xy() for basic
-       understanding. */
-    int diff_xy;
-    int diff_x, diff_y;
-    int width, height;
-
-    width = _mwidth(main_map_area); /* !! */
-    height = _mheight(main_map_area); /* !! */
-
-    map_x %= map.xsize;
-    if (map_x < map_view_x0) map_x += map.xsize;
-    diff_xy = (map_x + map_y) - (map_view_x0 + map_view_y0);
-    /* one diff_xy value defines a line parallel with the top of the isometric
-       view. */
-    *canvas_y = diff_xy/2 * NORMAL_TILE_HEIGHT + (diff_xy%2) * (NORMAL_TILE_HEIGHT/2);
-
-    /* changing both x and y with the same amount doesn't change the isometric
-       x value. (draw a grid to see it!) */
-    map_x -= diff_xy/2;
-    map_y -= diff_xy/2;
-    diff_x = map_x - map_view_x0;
-    diff_y = map_view_y0 - map_y;
-
-    *canvas_x = (diff_x - 1) * NORMAL_TILE_WIDTH
-      + (diff_x == diff_y ? NORMAL_TILE_WIDTH : NORMAL_TILE_WIDTH/2)
-      /* tiles starting above the visible area */
-      + (diff_y > diff_x ? NORMAL_TILE_WIDTH : 0);
-
-    /* We now have the corner of the sprite. For drawing we move it. */
-    *canvas_x -= NORMAL_TILE_WIDTH/2;
-
-    return (*canvas_x > -NORMAL_TILE_WIDTH)
-      && *canvas_x < (width + NORMAL_TILE_WIDTH/2)
-      && (*canvas_y > -NORMAL_TILE_HEIGHT)
-      && (*canvas_y < height);
-  } else { /* is_isometric */
-    if (map_view_x0+map_canvas_store_twidth <= map.xsize)
-      *canvas_x = map_x-map_view_x0;
-    else if(map_x >= map_view_x0)
-      *canvas_x = map_x-map_view_x0;
-    else if(map_x < map_adjust_x(map_view_x0+map_canvas_store_twidth))
-      *canvas_x = map_x+map.xsize-map_view_x0;
-    else *canvas_x = -1;
-
-    *canvas_y = map_y - map_view_y0;
-
-    *canvas_x *= NORMAL_TILE_WIDTH;
-    *canvas_y *= NORMAL_TILE_HEIGHT;
-
-    return *canvas_x >= 0
-      && *canvas_x < map_canvas_store_twidth*NORMAL_TILE_WIDTH
-      && *canvas_y >= 0
-      && *canvas_y < map_canvas_store_twidth*NORMAL_TILE_HEIGHT;
-  }
+  return map_pos_to_canvas_pos(map_x, map_y, canvas_x, canvas_y,
+			       map_view_x0, map_view_y0, width, height);
 }
 
 /**************************************************************************
 Finds the map coordinates corresponding to pixel coordinates.
+
+This function is almost identical between all GUI's.
 **************************************************************************/
 void get_map_xy(int canvas_x, int canvas_y, int *map_x, int *map_y)
 {
   int map_view_x0 = xget(main_map_area, MUIA_Map_HorizFirst);
   int map_view_y0 = xget(main_map_area, MUIA_Map_VertFirst);
 
-  if (is_isometric) {
-    *map_x = map_view_x0;
-    *map_y = map_view_y0;
-
-    /* first find an equivalent position on the left side of the screen. */
-    *map_x += canvas_x/NORMAL_TILE_WIDTH;
-    *map_y -= canvas_x/NORMAL_TILE_WIDTH;
-    canvas_x %= NORMAL_TILE_WIDTH;
-
-    /* Then move op to the top corner. */
-    *map_x += canvas_y/NORMAL_TILE_HEIGHT;
-    *map_y += canvas_y/NORMAL_TILE_HEIGHT;
-    canvas_y %= NORMAL_TILE_HEIGHT;
-
-    /* We are inside a rectangle, with 2 half tiles starting in the corner,
-       and two tiles further out. Draw a grid to see how this works :). */
-    assert(NORMAL_TILE_WIDTH == 2*NORMAL_TILE_HEIGHT);
-    canvas_y *= 2; /* now we have a square. */
-    if (canvas_x > canvas_y) (*map_y) -= 1;
-    if (canvas_x + canvas_y > NORMAL_TILE_WIDTH) (*map_x) += 1;
-
-    /* If we are outside the map find the nearest tile, with distance as
-       seen on the map. */
-    if (*map_y < 0) {
-      *map_y = 0;
-    } else if (*map_y >= map.ysize) {
-      *map_y = map.ysize - 1;
-    }
-
-    *map_x %= map.xsize;
-    if (*map_x < 0)
-      *map_x += map.xsize;
-  } else { /* is_isometric */
-    *map_x = map_adjust_x(map_view_x0 + canvas_x/NORMAL_TILE_WIDTH);
-    *map_y = map_adjust_y(map_view_y0 + canvas_y/NORMAL_TILE_HEIGHT);
-  }
+  canvas_pos_to_map_pos(canvas_x, canvas_y, map_x, map_y, map_view_x0,
+			map_view_y0);
 }
 
 /**************************************************************************
  GUI Independ (with new access functions)
 **************************************************************************/
-int tile_visible_mapcanvas(int x, int y)
+bool tile_visible_mapcanvas(int x, int y)
 {
   if (is_isometric) {
     int dummy_x, dummy_y; /* well, it needs two pointers... */
@@ -526,7 +402,7 @@ int tile_visible_mapcanvas(int x, int y)
 /**************************************************************************
  GUI Independ (with new access functions)
 **************************************************************************/
-int tile_visible_and_not_on_border_mapcanvas(int x, int y)
+bool tile_visible_and_not_on_border_mapcanvas(int x, int y)
 {
   if (is_isometric) {
     int canvas_x, canvas_y;
@@ -561,7 +437,7 @@ int tile_visible_and_not_on_border_mapcanvas(int x, int y)
 **************************************************************************/
 void move_unit_map_canvas(struct unit *punit, int x0, int y0, int dx, int dy)
 {
-  int dest_x, dest_y;
+  int dest_x, dest_y, is_real;
   /* only works for adjacent-square moves */
   if ((dx < -1) || (dx > 1) || (dy < -1) || (dy > 1) ||
       ((dx == 0) && (dy == 0))) {
@@ -573,8 +449,10 @@ void move_unit_map_canvas(struct unit *punit, int x0, int y0, int dx, int dy)
     update_unit_info_label(punit);
   }
 
-  dest_x = map_adjust_x(x0+dx);
-  dest_y = map_adjust_y(y0+dy);
+  dest_x = x0 + dx;
+  dest_y = y0 + dy;
+  is_real = normalize_map_pos(&dest_x, &dest_y);
+  assert(is_real);
 
   if (player_can_see_unit(game.player_ptr, punit) &&
       (tile_visible_mapcanvas(x0, y0) ||
@@ -608,34 +486,17 @@ void set_map_xy_start(int new_map_view_x0, int new_map_view_y0)
 }
 
 /**************************************************************************
-...
+Centers the mapview around (x, y).
+
+This function is almost identical between all GUI's.
 **************************************************************************/
 void center_tile_mapcanvas(int x, int y)
 {
   int map_view_x0;
   int map_view_y0;
-  int map_canvas_store_twidth = get_map_x_visible();
-  int map_canvas_store_theight = get_map_y_visible();
 
-  if (is_isometric) {
-    x -= map_canvas_store_twidth/2;
-    y += map_canvas_store_twidth/2;
-    x -= map_canvas_store_theight/2;
-    y -= map_canvas_store_theight/2;
-
-    map_view_x0 = map_adjust_x(x);
-    map_view_y0 = map_adjust_y(y);
-
-    map_view_y0 =
-      (map_view_y0 > map.ysize + EXTRA_BOTTOM_ROW - map_canvas_store_theight) ? 
-      map.ysize + EXTRA_BOTTOM_ROW - map_canvas_store_theight :
-      map_view_y0;
-  } else {
-    map_view_x0=map_adjust_x(x-map_canvas_store_twidth/2);
-    map_view_y0=map_adjust_y(y-map_canvas_store_theight/2);
-    if (map_view_y0>map.ysize+EXTRA_BOTTOM_ROW-map_canvas_store_theight)
-      map_view_y0=map_adjust_y(map.ysize+EXTRA_BOTTOM_ROW-map_canvas_store_theight);
-  }
+  base_center_tile_mapcanvas(x, y, &map_view_x0, &map_view_y0,
+			     get_map_x_visible(), get_map_y_visible());
 
   set_map_xy_start(map_view_x0, map_view_y0);
 // remove me
@@ -695,7 +556,7 @@ Works by first refreshing map_canvas_store and then drawing the result to
 the screen.
 **************************************************************************/
 void update_map_canvas(int x, int y, int width, int height, 
-		       int write_to_screen)
+		       bool write_to_screen)
 {
   DoMethod(main_map_area, MUIM_Map_Refresh, x, y, width, height, write_to_screen);
 }
@@ -718,10 +579,10 @@ void update_map_canvas_visible(void)
     width = height = map_canvas_store_twidth + map_canvas_store_theight;
     update_map_canvas(map_view_x0,
 		      map_view_y0 - map_canvas_store_twidth,
-		      width, height, 1);
+		      width, height, TRUE);
   } else {
     update_map_canvas(map_view_x0, map_view_y0,
-		      map_canvas_store_twidth,map_canvas_store_theight, 1);
+		      map_canvas_store_twidth,map_canvas_store_theight, TRUE);
   }
 
   show_city_descriptions();
@@ -776,4 +637,3 @@ void undraw_segment(int src_x, int src_y, int dir)
 {
   DoMethod(main_map_area, MUIM_Map_UndrawSegment, src_x, src_y, dir);
 }
-
