@@ -16,7 +16,6 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -48,7 +47,7 @@
 #include "support.h"
 
 #include "cityrep.h"
-#include "clinet.h"
+#include "citydlg.h"
 #include "colors.h"
 #include "control.h" /* request_xxx and set_unit_focus */
 #include "dialogs.h"
@@ -60,12 +59,14 @@
 #include "mapctrl.h"
 #include "mapview.h"
 #include "optiondlg.h"		/* for toggle_callback */
-#include "options.h"
 #include "repodlgs.h"
-#include "tilespec.h"
 #include "wldlg.h"
 
-#include "citydlg.h"
+#include "citydlg_common.h"
+#include "climisc.h"
+#include "clinet.h"
+#include "options.h"
+#include "tilespec.h"
 
 #include "cityicon.ico"
 
@@ -132,7 +133,7 @@ static int dialog_list_has_been_initialised;
 
 
 static struct city_dialog *get_city_dialog(struct city *pcity);
-static struct city_dialog *create_city_dialog(struct city *pcity, int make_modal);
+static struct city_dialog *create_city_dialog(struct city *pcity, bool make_modal);
 static void close_city_dialog(struct city_dialog *pdialog);
 
 static void city_dialog_update_improvement_list(struct city_dialog *pdialog);
@@ -283,35 +284,7 @@ static void get_contents_of_output(struct city_dialog *pdialog,
 static void get_contents_of_progress(struct city_dialog *pdialog,
 				     char *retbuf, int n)
 {
-  struct city *pcity = pdialog ? pdialog->pcity : NULL;
-  int stock=0;
-  int cost=0;
-  int turns=0;
-
-  if (pcity) {
-    stock = pcity->shield_stock;
-    if(pcity->is_building_unit) {
-      cost = get_unit_type(pcity->currently_building)->build_cost;
-      turns = city_turns_to_build(pcity, pcity->currently_building, TRUE);
-    } else {
-      cost = get_improvement_type(pcity->currently_building)->build_cost;
-      turns = city_turns_to_build(pcity, pcity->currently_building, FALSE);
-    }
-  }
-
-  if (pcity && (pcity->currently_building==B_CAPITAL)) {
-    /* Capitalization is special, you can't buy it or finish making it */
-    my_snprintf(retbuf, n,
-		concise_city_production ? " %3d/XXX:XXX " :
-		  _(" %3d/XXX XXX turns "),
-		stock);
-  } else {
-    my_snprintf(retbuf, n,
-		concise_city_production ? " %3d/%3d:%3d " :
-		  turns == 1 ? _(" %3d/%3d %3d turn ") :
-		    _(" %3d/%3d %3d turns "),
-		stock, cost, turns);
-  }
+  get_city_dialog_production(pdialog ? pdialog->pcity : NULL, retbuf, n);
 }
 
 /****************************************************************
@@ -322,7 +295,7 @@ static void get_contents_of_worklist(struct city_dialog *pdialog,
 {
   struct city *pcity = pdialog ? pdialog->pcity : NULL;
 
-  if (pcity && worklist_is_empty(pcity->worklist)) {
+  if (pcity && worklist_is_empty(&pcity->worklist)) {
     mystrlcpy(retbuf, _("(is empty)"), n);
   } else {
     mystrlcpy(retbuf, _("(in prog.)"), n);
@@ -353,13 +326,9 @@ struct city_dialog *get_city_dialog(struct city *pcity)
 /****************************************************************
 ...
 *****************************************************************/
-int city_dialog_is_open(struct city *pcity)
+bool city_dialog_is_open(struct city *pcity)
 {
-  if (get_city_dialog(pcity)) {
-    return 1;
-  } else {
-    return 0;
-  }
+  return get_city_dialog(pcity) != NULL;
 }
 
 /****************************************************************
@@ -431,7 +400,7 @@ void refresh_unit_city_dialogs(struct unit *punit)
 /****************************************************************
 popup the dialog 10% inside the main-window 
 *****************************************************************/
-void popup_city_dialog(struct city *pcity, int make_modal)
+void popup_city_dialog(struct city *pcity, bool make_modal)
 {
   struct city_dialog *pdialog;
   
@@ -487,7 +456,7 @@ static void city_map_canvas_expose(Widget w, XEvent *event, Region exposed,
 
 #define LAYOUT_DEBUG 0
 
-struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
+struct city_dialog *create_city_dialog(struct city *pcity, bool make_modal)
 {
   char *dummy_improvement_list[]={ 
     "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX",
@@ -528,12 +497,13 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
   pdialog->present_unit_base=0;
   pdialog->worklist_shell = NULL;
 
-  if(!icon_pixmap)
-    icon_pixmap=
-      XCreateBitmapFromData(display,
-			    RootWindowOfScreen(XtScreen(toplevel)),
-			    cityicon_bits,
-			    cityicon_width, cityicon_height);
+  if (!icon_pixmap) {
+    icon_pixmap =
+	XCreateBitmapFromData(display,
+			      RootWindowOfScreen(XtScreen(toplevel)),
+			      (char *) cityicon_bits,
+			      cityicon_width, cityicon_height);
+  }
 
 
   pdialog->shell=
@@ -1398,8 +1368,7 @@ static void rename_city_callback(Widget w, XtPointer client_data,
 
   if((pdialog=(struct city_dialog *)client_data)) {
     packet.city_id=pdialog->pcity->id;
-    packet.worklist.name[0] = '\0';
-    sz_strlcpy(packet.name, input_dialog_get_input(w));
+     sz_strlcpy(packet.name, input_dialog_get_input(w));
     send_packet_city_request(&aconnection, &packet, PACKET_CITY_RENAME);
   }
   input_dialog_destroy(w);
@@ -1453,7 +1422,7 @@ void trade_callback(Widget w, XtPointer client_data, XtPointer call_data)
 	      pdialog->pcity->name);
   bptr = end_of_strn(bptr, &nleft);
   
-  for(i=0; i<4; i++)
+  for (i = 0; i < NUM_TRADEROUTES; i++)
     if(pdialog->pcity->trade[i]) {
       struct city *pcity;
       x=1;
@@ -1561,32 +1530,33 @@ void city_dialog_update_map(struct city_dialog *pdialog)
   int x, y;
   struct city *pcity=pdialog->pcity;
   
-  for(y=0; y<CITY_MAP_SIZE; y++)
+  for(y=0; y<CITY_MAP_SIZE; y++) {
     for(x=0; x<CITY_MAP_SIZE; x++) {
-      if(!(x==0 && y==0) && !(x==0 && y==CITY_MAP_SIZE-1) &&
-	 !(x==CITY_MAP_SIZE-1 && y==0) && 
-	 !(x==CITY_MAP_SIZE-1 && y==CITY_MAP_SIZE-1) &&
-	 tile_is_known(pcity->x+x-CITY_MAP_SIZE/2, 
-		       pcity->y+y-CITY_MAP_SIZE/2)) {
-	pixmap_put_tile(XtWindow(pdialog->map_canvas),
-	                pcity->x+x-CITY_MAP_SIZE/2,
-			pcity->y+y-CITY_MAP_SIZE/2,
-			x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT, 1);
-	if(pcity->city_map[x][y]==C_TILE_WORKER)
+      int map_x, map_y;
+
+      if (is_valid_city_coords(x, y)
+	  && city_map_to_map(&map_x, &map_y, pcity, x, y)
+	  && tile_get_known(map_x, map_y)) {
+	pixmap_put_tile(XtWindow(pdialog->map_canvas), map_x, map_y,
+			x * NORMAL_TILE_WIDTH, y * NORMAL_TILE_HEIGHT, 1);
+	if (pcity->city_map[x][y] == C_TILE_WORKER)
 	  put_city_tile_output(XtWindow(pdialog->map_canvas),
-			       x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT,
+			       x * NORMAL_TILE_WIDTH,
+			       y * NORMAL_TILE_HEIGHT,
 			       city_get_food_tile(x, y, pcity),
-			       city_get_shields_tile(x, y, pcity), 
-			       city_get_trade_tile(x, y, pcity) );
-	else if(pcity->city_map[x][y]==C_TILE_UNAVAILABLE)
+			       city_get_shields_tile(x, y, pcity),
+			       city_get_trade_tile(x, y, pcity));
+	else if (pcity->city_map[x][y] == C_TILE_UNAVAILABLE)
 	  pixmap_frame_tile_red(XtWindow(pdialog->map_canvas),
-				x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT);
-      }
-      else {
+				x * NORMAL_TILE_WIDTH,
+				y * NORMAL_TILE_HEIGHT);
+      } else {
 	pixmap_put_black_tile(XtWindow(pdialog->map_canvas),
-			      x*NORMAL_TILE_WIDTH, y*NORMAL_TILE_HEIGHT);
+			      x * NORMAL_TILE_WIDTH,
+			      y * NORMAL_TILE_HEIGHT);
       }
     }
+  }
 }
 
 /****************************************************************
@@ -1629,6 +1599,18 @@ void city_dialog_update_citizens(struct city_dialog *pdialog)
     }
   if(n<pcity->ppl_unhappy[4]) goto city_dialog_update_citizens_overflow;
 
+  for (n = 0; n < pcity->ppl_angry[4] && i < pdialog->num_citizens_shown;
+       n++, i++)
+    if (pdialog->citizen_type[i] != 9 && pdialog->citizen_type[i] != 10) {
+      pdialog->citizen_type[i] = 9 + i % 2;
+      xaw_set_bitmap(pdialog->citizen_labels[i],
+		     get_citizen_pixmap(pdialog->citizen_type[i]));
+      XtRemoveAllCallbacks(pdialog->citizen_labels[i], XtNcallback);
+      XtSetSensitive(pdialog->citizen_labels[i], FALSE);
+    }
+  if (n < pcity->ppl_angry[4])
+    goto city_dialog_update_citizens_overflow;
+
   for(n=0; n<pcity->ppl_elvis && i<pdialog->num_citizens_shown; n++, i++)
     if(pdialog->citizen_type[i]!=0) {
       xaw_set_bitmap(pdialog->citizen_labels[i], get_citizen_pixmap(0));
@@ -1802,8 +1784,8 @@ void city_dialog_update_supported_units(struct city_dialog *pdialog,
     xaw_expose_now(pixcomm);
 
     XtRemoveAllCallbacks(pixcomm, XtNcallback);
-    XtAddCallback(pixcomm, XtNcallback, 
-		  support_units_callback, (XtPointer)punit->id);
+    XtAddCallback(pixcomm, XtNcallback,
+		  support_units_callback, INT_TO_XTPOINTER(punit->id));
     XtSetSensitive(pixcomm, TRUE);
   }
 
@@ -1863,7 +1845,7 @@ void city_dialog_update_present_units(struct city_dialog *pdialog, int unitid)
 
     XtRemoveAllCallbacks(pixcomm, XtNcallback);
     XtAddCallback(pixcomm, XtNcallback, 
-		  present_units_callback, (XtPointer)punit->id);
+		  present_units_callback, INT_TO_XTPOINTER(punit->id));
     XtSetSensitive(pixcomm, TRUE);
   }
 
@@ -1883,10 +1865,11 @@ void city_dialog_update_title(struct city_dialog *pdialog)
   String now;
   
   my_snprintf(buf, sizeof(buf), _("%s - %s citizens"),
-	  pdialog->pcity->name, int_to_text(city_population(pdialog->pcity)));
+	      pdialog->pcity->name,
+	      population_to_text(city_population(pdialog->pcity)));
 
   XtVaGetValues(pdialog->cityname_label, XtNlabel, &now, NULL);
-  if(strcmp(now, buf)) {
+  if(strcmp(now, buf) != 0) {
     XtVaSetValues(pdialog->cityname_label, XtNlabel, (XtArgVal)buf, NULL);
     xaw_horiz_center(pdialog->cityname_label);
     XtVaSetValues(pdialog->shell, XtNtitle, (XtArgVal)pdialog->pcity->name, NULL);
@@ -1898,18 +1881,18 @@ void city_dialog_update_title(struct city_dialog *pdialog)
 *****************************************************************/
 void city_dialog_update_improvement_list(struct city_dialog *pdialog)
 {
-  int i, n, flag;
+  int n = 0, flag = 0;
 
-  for(i=0, n=0, flag=0; i<game.num_impr_types; ++i)
-    if(pdialog->pcity->improvements[i]) {
-      if(!pdialog->improvlist_names_ptrs[n] ||
-	 strcmp(pdialog->improvlist_names_ptrs[n], get_impr_name_ex(pdialog->pcity, i)))
-	flag=1;
-      sz_strlcpy(pdialog->improvlist_names[n],
-		 get_impr_name_ex(pdialog->pcity, i));
-      pdialog->improvlist_names_ptrs[n]=pdialog->improvlist_names[n];
-      n++;
-    }
+  built_impr_iterate(pdialog->pcity, i) {
+    if (!pdialog->improvlist_names_ptrs[n] ||
+	strcmp(pdialog->improvlist_names_ptrs[n],
+	       get_impr_name_ex(pdialog->pcity, i)) != 0)
+      flag = 1;
+    sz_strlcpy(pdialog->improvlist_names[n],
+	       get_impr_name_ex(pdialog->pcity, i));
+    pdialog->improvlist_names_ptrs[n] = pdialog->improvlist_names[n];
+    n++;
+  } built_impr_iterate_end;
   
   if(pdialog->improvlist_names_ptrs[n]!=0) {
     pdialog->improvlist_names_ptrs[n]=0;
@@ -1951,8 +1934,6 @@ void citydlg_btn_select_citymap(Widget w, XEvent *event)
     packet.city_id=pcity->id;
     packet.worker_x=xtile;
     packet.worker_y=ytile;
-    packet.name[0]='\0';
-    packet.worklist.name[0] = '\0';
     
     if(pcity->city_map[xtile][ytile]==C_TILE_WORKER)
       send_packet_city_request(&aconnection, &packet, 
@@ -1973,8 +1954,6 @@ void elvis_callback(Widget w, XtPointer client_data, XtPointer call_data)
   pdialog=(struct city_dialog *)client_data;
 
   packet.city_id=pdialog->pcity->id;
-  packet.name[0]='\0';
-  packet.worklist.name[0] = '\0';
   packet.specialist_from=SP_ELVIS;
   packet.specialist_to=SP_SCIENTIST;
   
@@ -1993,8 +1972,6 @@ void scientist_callback(Widget w, XtPointer client_data, XtPointer call_data)
   pdialog=(struct city_dialog *)client_data;
 
   packet.city_id=pdialog->pcity->id;
-  packet.name[0]='\0';
-  packet.worklist.name[0] = '\0';
   packet.specialist_from=SP_SCIENTIST;
   packet.specialist_to=SP_TAXMAN;
   
@@ -2013,8 +1990,6 @@ void taxman_callback(Widget w, XtPointer client_data, XtPointer call_data)
   pdialog=(struct city_dialog *)client_data;
 
   packet.city_id=pdialog->pcity->id;
-  packet.name[0]='\0';
-  packet.worklist.name[0] = '\0';
   packet.specialist_from=SP_TAXMAN;
   packet.specialist_to=SP_ELVIS;
   
@@ -2034,8 +2009,6 @@ static void buy_callback_yes(Widget w, XtPointer client_data,
   pdialog=(struct city_dialog *)client_data;
 
   packet.city_id=pdialog->pcity->id;
-  packet.name[0]='\0';
-  packet.worklist.name[0] = '\0';
   send_packet_city_request(&aconnection, &packet, PACKET_CITY_BUY);
 
   destroy_message_dialog(w);
@@ -2059,7 +2032,7 @@ void buy_callback(Widget w, XtPointer client_data, XtPointer call_data)
 {
   struct city_dialog *pdialog;
   int value;
-  char *name;
+  const char *name;
   char buf[512];
   
   pdialog=(struct city_dialog *)client_data;
@@ -2148,10 +2121,10 @@ void upgrade_callback(Widget w, XtPointer client_data, XtPointer call_data)
 		      "Treasury contains %d gold."),
 		    unit_types[ut1].name, unit_types[ut2].name,
 		    value, game.player_ptr->economic.gold);
-        popup_message_dialog(toplevel, "upgradedialog", buf,
-                             unitupgrade_callback_yes, (XtPointer)(punit->id), 0,
-                             unitupgrade_callback_no, 0, 0,
-                             NULL);
+	popup_message_dialog(toplevel, "upgradedialog", buf,
+			     unitupgrade_callback_yes,
+			     INT_TO_XTPOINTER(punit->id), 0,
+			     unitupgrade_callback_no, 0, 0, NULL);
       } else {
         my_snprintf(buf, sizeof(buf),
 		    _("Upgrading %s to %s costs %d gold.\n"
@@ -2185,8 +2158,6 @@ static void change_to_callback(Widget w, XtPointer client_data,
     struct packet_city_request packet;
   
     packet.city_id=pdialog->pcity->id;
-    packet.name[0]='\0';
-    packet.worklist.name[0] = '\0';
     packet.build_id=pdialog->change_list_ids[ret->list_index];
     packet.is_build_id_unit_id=
       (ret->list_index >= pdialog->change_list_num_improvements);
@@ -2226,7 +2197,7 @@ static void change_help_callback(Widget w, XtPointer client_data,
   ret=XawListShowCurrent(pdialog->change_list);
   if(ret->list_index!=XAW_LIST_NONE) {
     int idx = pdialog->change_list_ids[ret->list_index];
-    int is_unit = (ret->list_index >= pdialog->change_list_num_improvements);
+    bool is_unit = (ret->list_index >= pdialog->change_list_num_improvements);
 
     if (is_unit) {
       popup_help_dialog_typed(get_unit_type(idx)->name, HELP_UNIT);
@@ -2269,7 +2240,7 @@ void change_callback(Widget w, XtPointer client_data, XtPointer call_data)
   Position x, y;
   Dimension width, height;
   struct city_dialog *pdialog;
-  int i, n, turns;
+  int n;
   
   pdialog=(struct city_dialog *)client_data;
   
@@ -2340,41 +2311,29 @@ void change_callback(Widget w, XtPointer client_data, XtPointer call_data)
   
   XtSetSensitive(pdialog->shell, FALSE);
 
-  for(i=0, n=0; i<game.num_impr_types; i++)
+  n = 0;
+  impr_type_iterate(i) {
     if(can_build_improvement(pdialog->pcity, i)) {
-      if (i==B_CAPITAL) {
-	my_snprintf(pdialog->change_list_names[n],
-		    sizeof(pdialog->change_list_names[n]),
-		    "%s (XX)",
-		    get_impr_name_ex(pdialog->pcity, i));
-      } else {
-	turns = city_turns_to_build (pdialog->pcity, i, FALSE);
-	my_snprintf(pdialog->change_list_names[n],
-		    sizeof(pdialog->change_list_names[n]),
-		    turns == 1 ? _("%s (%d) %d turn") : _("%s (%d) %d turns"),
-		    get_impr_name_ex(pdialog->pcity, i),
-		    get_improvement_type(i)->build_cost,
-		    turns);
-      }
+      get_city_dialog_production_full(pdialog->change_list_names[n],
+                                      sizeof(pdialog->change_list_names[n]),
+                                      i, FALSE, pdialog->pcity);
       pdialog->change_list_names_ptrs[n]=pdialog->change_list_names[n];
       pdialog->change_list_ids[n++]=i;
     }
+  } impr_type_iterate_end;
   
   pdialog->change_list_num_improvements=n;
 
 
-  for(i=0; i<game.num_unit_types; i++)
+  unit_type_iterate(i) {
     if(can_build_unit(pdialog->pcity, i)) {
-      turns = city_turns_to_build (pdialog->pcity, i, TRUE);
-      my_snprintf(pdialog->change_list_names[n],
-		  sizeof(pdialog->change_list_names[n]),
-		  turns == 1 ? _("%s (%d) %d turn") : _("%s (%d) %d turns"),
-		  get_unit_name(i),
-		  get_unit_type(i)->build_cost,
-		  turns);
+      get_city_dialog_production_full(pdialog->change_list_names[n],
+                                      sizeof(pdialog->change_list_names[n]),
+                                      i, TRUE, pdialog->pcity);
       pdialog->change_list_names_ptrs[n]=pdialog->change_list_names[n];
       pdialog->change_list_ids[n++]=i;
     }
+  } unit_type_iterate_end;
   
   pdialog->change_list_names_ptrs[n]=0;
 
@@ -2399,8 +2358,8 @@ void worklist_callback(Widget w, XtPointer client_data, XtPointer call_data)
     XtPopup(pdialog->worklist_shell, XtGrabNone);
   } else {
     pdialog->worklist_shell = 
-      popup_worklist(pdialog->pcity->worklist, pdialog->pcity, pdialog->shell, 
-		     (void *)pdialog, commit_city_worklist,
+      popup_worklist(&pdialog->pcity->worklist, pdialog->pcity,
+		     pdialog->shell, (void *) pdialog, commit_city_worklist,
 		     cancel_city_worklist);
   }
 }
@@ -2412,7 +2371,8 @@ void commit_city_worklist(struct worklist *pwl, void *data)
 {
   struct packet_city_request packet;
   struct city_dialog *pdialog = (struct city_dialog *)data;
-  int i, k, id, is_unit;
+  int k, id;
+  bool is_unit;
 
   /* Update the worklist.  Remember, though -- the current build 
      target really isn't in the worklist; don't send it to the 
@@ -2445,8 +2405,6 @@ void commit_city_worklist(struct worklist *pwl, void *data)
 
 	/* Change the current target */
 	packet.city_id=pdialog->pcity->id;
-	packet.name[0] = '\0';
-	packet.worklist.name[0] = '\0';
 	packet.build_id = id;
 	packet.is_build_id_unit_id = is_unit;
 	send_packet_city_request(&aconnection, &packet, PACKET_CITY_CHANGE);
@@ -2461,14 +2419,8 @@ void commit_city_worklist(struct worklist *pwl, void *data)
 
   /* Send the rest of the worklist on its way. */
   packet.city_id=pdialog->pcity->id;
-  packet.name[0] = '\0';
+  copy_worklist(&packet.worklist, pwl);
   packet.worklist.name[0] = '\0';
-  packet.worklist.is_valid = 1;
-  for (i = 0; i < MAX_LEN_WORKLIST; i++) {
-    packet.worklist.wlefs[i] = pwl->wlefs[i];
-    packet.worklist.wlids[i] = pwl->wlids[i];
-  }
-    
   send_packet_city_request(&aconnection, &packet, PACKET_CITY_WORKLIST);
 
   pdialog->worklist_shell = NULL;
@@ -2493,8 +2445,6 @@ static void sell_callback_yes(Widget w, XtPointer client_data,
 
   packet.city_id=pdialog->pcity->id;
   packet.build_id=pdialog->sell_id;
-  packet.name[0]='\0';
-  packet.worklist.name[0] = '\0';
   send_packet_city_request(&aconnection, &packet, PACKET_CITY_SELL);
 
   destroy_message_dialog(w);
@@ -2524,29 +2474,28 @@ void sell_callback(Widget w, XtPointer client_data, XtPointer call_data)
   ret=XawListShowCurrent(pdialog->improvement_list);
 
   if(ret->list_index!=XAW_LIST_NONE) {
-    int i, n;
-    for(i=0, n=0; i<game.num_impr_types; i++)
-      if(pdialog->pcity->improvements[i]) {
-	if(n==ret->list_index) {
-	  char buf[512];
-	  
-	  if(is_wonder(i))
-	    return;
-	  
-	  pdialog->sell_id=i;
-	  my_snprintf(buf, sizeof(buf), _("Sell %s for %d gold?"), 
-		      get_impr_name_ex(pdialog->pcity, i),
-		      improvement_value(i));
+    int n = 0;
+    built_impr_iterate(pdialog->pcity, i) {
+      if (n == ret->list_index) {
+	char buf[512];
 
-	  popup_message_dialog(pdialog->shell, "selldialog", buf,
-			       sell_callback_yes, pdialog, 0,
-			       sell_callback_no, pdialog, 0,
-			       NULL);
-	  
+	if (is_wonder(i)) {
 	  return;
 	}
-	n++;
+
+	pdialog->sell_id = i;
+	my_snprintf(buf, sizeof(buf), _("Sell %s for %d gold?"),
+		    get_impr_name_ex(pdialog->pcity, i),
+		    improvement_value(i));
+
+	popup_message_dialog(pdialog->shell, "selldialog", buf,
+			     sell_callback_yes, pdialog, 0,
+			     sell_callback_no, pdialog, 0, NULL);
+
+	return;
       }
+      n++;
+    } built_impr_iterate_end;
   }
 }
 
@@ -2651,7 +2600,7 @@ void cityopt_callback(Widget w, XtPointer client_data,
 {
   struct city_dialog *pdialog = (struct city_dialog *)client_data;
   struct city *pcity = pdialog->pcity;
-  int i, state;
+  int i;
 
   if(cityopt_shell) {
     XtDestroyWidget(cityopt_shell);
@@ -2659,13 +2608,13 @@ void cityopt_callback(Widget w, XtPointer client_data,
   cityopt_shell=create_cityopt_dialog(pcity->name);
   /* Doing this here makes the "No"'s centered consistently */
   for(i=0; i<NUM_CITYOPT_TOGGLES; i++) {
-    state = (pcity->city_options & (1<<i));
+    bool state = is_city_option_set(pcity, i);
     XtVaSetValues(cityopt_toggles[i], XtNstate, state,
 		  XtNlabel, state?_("Yes"):_("No"), NULL);
   }
-  if (pcity->city_options & (1<<CITYO_NEW_EINSTEIN)) {
+  if (is_city_option_set(pcity, CITYO_NEW_EINSTEIN)) {
     newcitizen_index = 1;
-  } else if (pcity->city_options & (1<<CITYO_NEW_TAXMAN)) {
+  } else if (is_city_option_set(pcity, CITYO_NEW_TAXMAN)) {
     newcitizen_index = 2;
   } else {
     newcitizen_index = 0;
@@ -2840,4 +2789,3 @@ void popdown_cityopt_dialog(void)
     cityopt_shell = 0;
   }
 }
-
