@@ -17,7 +17,6 @@
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
 #include <stdarg.h>
 
 #include <X11/Intrinsic.h>
@@ -62,6 +61,7 @@ static Widget races_dialog_shell=NULL;
 static Widget races_form, races_toggles_form, races_label;
 static Widget *races_toggles=NULL;
 static int *races_toggles_to_nations=NULL;
+static int *nation_to_race_toggle = NULL;
 static Widget races_leader_form, races_leader;
 static Widget races_leader_pick_popupmenu, races_leader_pick_menubutton;
 static Widget races_sex_toggles[2], races_sex_form, races_sex_label;
@@ -178,7 +178,21 @@ static void notify_command_callback(Widget w, XtPointer client_data,
   XtSetSensitive(toplevel, TRUE);
 }
 
+/****************************************************************
+ ...
+*****************************************************************/
+static void select_random_race(void)
+{
+  /* try to find a free nation */
+  while (1) {
+    int race_toggle_index = myrand(game.playable_nation_count);
 
+    if (XtIsSensitive(races_toggles[race_toggle_index])) {
+      x_simulate_button_click(races_toggles[race_toggle_index]);
+      break;
+    }
+  }
+}
 
 /****************************************************************
 ...
@@ -311,7 +325,7 @@ void popup_notify_goto_dialog(char *headline, char *lines,int x, int y)
   Widget notify_headline, notify_label;
   Dimension width, width2, width_1, width_2;
   
-  if (y < 0 || y >= map.ysize) {
+  if (!is_real_tile(x, y)) {
     popup_notify_dialog("Message:", headline, lines);
     return;
   }
@@ -694,7 +708,7 @@ static void spy_sabotage_callback(Widget w, XtPointer client_data,
 ...
 *****************************************************************/
 static int create_advances_list(struct player *pplayer,
-				struct player *pvictim, int make_modal)
+				struct player *pvictim, bool make_modal)
 {  
   Widget spy_tech_form;
   Widget close_command;
@@ -779,14 +793,14 @@ static int create_advances_list(struct player *pplayer,
 ...
 *****************************************************************/
 static int create_improvements_list(struct player *pplayer,
-				    struct city *pcity, int make_modal)
+				    struct city *pcity, bool make_modal)
 {  
   Widget spy_sabotage_form;
   Widget close_command;
   Dimension width1, width2; 
-  int i, j;
+  int j;
 
-  static char *improvements_can_sabotage[B_LAST+1]; 
+  static const char *improvements_can_sabotage[B_LAST+1]; 
   
   spy_sabotage_shell =
     I_T(XtVaCreatePopupShell("spysabotageimprovementspopup", 
@@ -835,11 +849,12 @@ static int create_improvements_list(struct player *pplayer,
   improvements_can_sabotage[j] = _("City Production");
   improvement_type[j++] = -1;
 
-  for(i=0; i<game.num_impr_types; i++) 
-    if(i != B_PALACE && pcity->improvements[i] && !is_wonder(i)) {
+  built_impr_iterate(pcity, i) {
+    if (i != B_PALACE && !is_wonder(i)) {
       improvements_can_sabotage[j] = get_impr_name_ex(pcity, i);
       improvement_type[j++] = i;
     }  
+  } built_impr_iterate_end;
 
   if(j > 1) {
     improvements_can_sabotage[j] = _("At Spy's Discretion");
@@ -852,7 +867,8 @@ static int create_improvements_list(struct player *pplayer,
   
   XtSetSensitive(spy_sabotage_command, FALSE);
   
-  XawListChange(spy_improvements_list, improvements_can_sabotage, 0, 0, 1);
+  XawListChange(spy_improvements_list, (String *) improvements_can_sabotage,
+		0, 0, 1);
   XtVaGetValues(spy_improvements_list, XtNwidth, &width1, NULL);
   XtVaGetValues(spy_improvements_list_label, XtNwidth, &width2, NULL);
   XtVaSetValues(spy_improvements_list, XtNwidth, MAX(width1,width2), NULL); 
@@ -999,7 +1015,12 @@ void popup_incite_dialog(struct city *pcity)
 {
   char buf[128];
 
-  if(game.player_ptr->economic.gold>=pcity->incite_revolt_cost) {
+  if (pcity->incite_revolt_cost == INCITE_IMPOSSIBLE_COST) {
+    my_snprintf(buf, sizeof(buf), _("You can't incite a revolt in %s."),
+		pcity->name);
+    popup_message_dialog(toplevel, "diplomatnogolddialog", buf,
+			 diplomat_incite_no_callback, 0, 0, NULL);
+  } else if (game.player_ptr->economic.gold >= pcity->incite_revolt_cost) {
     my_snprintf(buf, sizeof(buf),
 		_("Incite a revolt for %d gold?\n"
 		  "Treasury contains %d gold."), 
@@ -1078,7 +1099,7 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
 		_("Your %s has arrived at %s.\nWhat is your command?"),
 		unit_name(punit->type), pcity->name);
 
-    if(!unit_flag(punit->type, F_SPY)){
+    if(!unit_flag(punit, F_SPY)){
       shl=popup_message_dialog(toplevel, "diplomatdialog", buf,
 			       diplomat_embassy_callback, 0, 1,
 			       diplomat_investigate_callback, 0, 1,
@@ -1134,7 +1155,7 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
     if((ptunit=unit_list_get(&map_get_tile(dest_x, dest_y)->units, 0))){
       /* Spy/Diplomat acting against a unit */
       
-      char *message = (!unit_flag(punit->type, F_SPY))?
+      char *message = (!unit_flag(punit, F_SPY))?
 	_("Sir, the diplomat is waiting for your command"):
 	_("Sir, the spy is waiting for your command");
       
@@ -1309,7 +1330,8 @@ void popup_government_dialog(void)
 				     XtNfromVert, prev,
 				     XtNlabel, (XtArgVal)governments[i].name,
 				     NULL);
-    XtAddCallback(button, XtNcallback, government_callback, (XtPointer)i);
+    XtAddCallback(button, XtNcallback, government_callback,
+		  INT_TO_XTPOINTER(i));
     XtSetSensitive(button, can_change ? TRUE : FALSE);
     prev = button;
   }
@@ -1370,9 +1392,8 @@ static void pillage_callback(Widget w, XtPointer client_data,
   if (client_data) {
     struct unit *punit = find_unit_by_id (unit_to_use_to_pillage);
     if (punit) {
-      request_new_unit_activity_targeted (punit,
-					  ACTIVITY_PILLAGE,
-					  (int)client_data);
+      request_new_unit_activity_targeted(punit, ACTIVITY_PILLAGE,
+					 XTPOINTER_TO_INT(client_data));
     }
   }
 
@@ -1383,10 +1404,10 @@ static void pillage_callback(Widget w, XtPointer client_data,
 /****************************************************************
 ...
 *****************************************************************/
-void popup_pillage_dialog(struct unit *punit, int may_pillage)
+void popup_pillage_dialog(struct unit *punit,
+			  enum tile_special_type may_pillage)
 {
   Widget shell, form, dlabel, button, prev;
-  int what;
 
   if (is_showing_pillage_dialog) {
     return;
@@ -1403,14 +1424,16 @@ void popup_pillage_dialog(struct unit *punit, int may_pillage)
 
   prev = dlabel;
   while (may_pillage) {
-    what = get_preferred_pillage (may_pillage);
+    enum tile_special_type what = get_preferred_pillage(may_pillage);
+
     button =
       XtVaCreateManagedWidget ("button", commandWidgetClass, form,
 			       XtNfromVert, prev,
 			       XtNlabel,
 			         (XtArgVal)(map_get_infrastructure_text (what)),
 			       NULL);
-    XtAddCallback (button, XtNcallback, pillage_callback, (XtPointer)what);
+    XtAddCallback(button, XtNcallback, pillage_callback,
+		  INT_TO_XTPOINTER(what));
     prev = button;
     may_pillage &= (~(what | map_get_infrastructure_prerequisite (what)));
   }
@@ -1431,7 +1454,7 @@ static void unit_connect_callback (Widget w, XtPointer client_data,
 				   XtPointer call_data)
 {
   struct unit *punit;
-  int activity = (int)client_data;
+  int activity = XTPOINTER_TO_INT(client_data);
 
   if (!is_showing_unit_connect_dialog) {
     destroy_message_dialog(w);
@@ -1491,8 +1514,8 @@ void popup_unit_connect_dialog(struct unit *punit, int dest_x, int dest_y)
 			       XtNlabel,
 			         (XtArgVal)(get_activity_text (activity)),
 			       NULL);
-    XtAddCallback (button, XtNcallback, unit_connect_callback, 
-		   (XtPointer)activity);
+    XtAddCallback(button, XtNcallback, unit_connect_callback,
+		  INT_TO_XTPOINTER(activity));
     prev = button;
   }
   button =
@@ -1509,7 +1532,8 @@ void popup_unit_connect_dialog(struct unit *punit, int dest_x, int dest_y)
   Parameters after named parameters should be in triplets:
   - callback, callback_data, fixed_width 
 *****************************************************************/
-Widget popup_message_dialog(Widget parent, char *dialogname, char *text, ...)
+Widget popup_message_dialog(Widget parent, const char *dialogname,
+			    const char *text, ...)
 {
   va_list args;
   Widget dshell, dform, button;
@@ -1629,7 +1653,7 @@ void popup_unit_select_dialog(struct tile *ptile)
 
   for(i=0; i<n; i++) {
     struct unit *punit=unit_list_get(&ptile->units, i);
-    struct unit_type *punittemp=get_unit_type(punit->type);
+    struct unit_type *punittemp=unit_type(punit);
     struct city *pcity;
     
     if(!(i%r))  {
@@ -2056,8 +2080,8 @@ void create_races_dialog(void)
 
 
   for(i=0; i<game.playable_nation_count; i++) {
-    XtAddCallback(races_toggles[i], XtNcallback, 
-		  races_toggles_callback, (XtPointer)i);
+    XtAddCallback(races_toggles[i], XtNcallback,
+		  races_toggles_callback, INT_TO_XTPOINTER(i));
   }
 
 
@@ -2073,6 +2097,15 @@ void create_races_dialog(void)
     races_toggles_to_nations[i] = i;
   }
   qsort(races_toggles_to_nations, i, sizeof(int), races_indirect_compare);
+
+  /* Build nation_to_race_toggle */
+  free(nation_to_race_toggle);
+  nation_to_race_toggle =
+      fc_calloc(game.playable_nation_count, sizeof(int));
+  for (i = 0; i < game.playable_nation_count; i++) {
+    nation_to_race_toggle[races_toggles_to_nations[i]] = i;
+  }
+
   for(i=0; i<game.playable_nation_count; i++) {
     XtVaSetValues (races_toggles[i],
 		   XtNlabel, (XtArgVal)get_nation_name(races_toggles_to_nations[i]),
@@ -2084,7 +2117,7 @@ void create_races_dialog(void)
 		  (XtArgVal)city_styles[city_style_idx[i]].name, NULL);
   }
 
-  x_simulate_button_click(races_toggles[myrand(game.playable_nation_count)]);
+  select_random_race();
 }
 
 /****************************************************************
@@ -2100,24 +2133,34 @@ void racesdlg_key_ok(Widget w)
 /**************************************************************************
 ...
 **************************************************************************/
-void races_toggles_set_sensitive(int bits1, int bits2)
+void races_toggles_set_sensitive(struct packet_nations_used *packet)
 {
-  int i, race, selected;
-  Boolean sens;
+  int i;
 
-  for(i=0; i<game.playable_nation_count; i++) {
-    race = races_toggles_to_nations[i];
-    sens = !( ( ((race<32)?bits1:bits2) >> ((race<32)?race:race-32) )&1 );
-    XtSetSensitive(races_toggles[i], sens);
+  for (i = 0; i < game.playable_nation_count; i++) {
+    XtSetSensitive(races_toggles[nation_to_race_toggle[i]], TRUE);
   }
 
-  if((selected=races_buttons_get_current())==-1)
-    return;
-  race = races_toggles_to_nations[selected];
+  freelog(LOG_DEBUG, "%d nations used:", packet->num_nations_used);
+  for (i = 0; i < packet->num_nations_used; i++) {
+    int nation = packet->nations_used[i], selected_nation = -1;
 
-  if( (bits1 & (1<<race)) || 
-      (race>32 && (bits2 & (1<<(race-32))) ) )
-    XawToggleUnsetCurrent(races_toggles[0]);
+    if (races_buttons_get_current() != -1) {
+      selected_nation =
+	  races_toggles_to_nations[races_buttons_get_current()];
+    }
+
+    freelog(LOG_DEBUG, "  [%d]: %d = %s", i, nation,
+	    get_nation_name(nation));
+
+    if (nation == selected_nation) {
+      XawToggleUnsetCurrent(races_toggles[0]);
+      XtSetSensitive(races_toggles[nation_to_race_toggle[nation]], FALSE);
+      select_random_race();
+    } else {
+      XtSetSensitive(races_toggles[nation_to_race_toggle[nation]], FALSE);
+    }
+  }
 }
 
 /**************************************************************************
@@ -2126,17 +2169,13 @@ void races_toggles_set_sensitive(int bits1, int bits2)
 void races_toggles_callback(Widget w, XtPointer client_data, 
 			    XtPointer call_data)
 {
-  int index, race;
+  int index = XTPOINTER_TO_INT(client_data);
+  int race = races_toggles_to_nations[index];
   int j;
   int leader_count;
-  char **leaders;
+  char **leaders = get_nation_leader_names(race, &leader_count);
   Widget entry;
 
-  index = (int)client_data;
-  race = races_toggles_to_nations[index];
-
-  leaders = get_nation_leader_names(race, &leader_count);
-
   if(races_leader_pick_popupmenu)
     XtDestroyWidget(races_leader_pick_popupmenu);
 
@@ -2152,10 +2191,8 @@ void races_toggles_callback(Widget w, XtPointer client_data,
 			      smeBSBObjectClass,
 			      races_leader_pick_popupmenu,
 			      NULL);
-    XtAddCallback(entry,
-		  XtNcallback,
-		  races_leader_pick_callback,
-		  (XtPointer)((MAX_NUM_NATIONS * race) + j));
+    XtAddCallback(entry, XtNcallback, races_leader_pick_callback,
+		  INT_TO_XTPOINTER(MAX_NUM_NATIONS * race + j));
   }
 
   races_leader_set_values(race, myrand(leader_count));
@@ -2172,10 +2209,8 @@ void races_toggles_callback(Widget w, XtPointer client_data,
 void races_leader_pick_callback(Widget w, XtPointer client_data,
 				XtPointer call_data)
 {
-  int race, lead;
-
-  race = ((int)client_data) / MAX_NUM_NATIONS;
-  lead = ((int)client_data) - (MAX_NUM_NATIONS * race);
+  int race = XTPOINTER_TO_INT(client_data) / MAX_NUM_NATIONS;
+  int lead = XTPOINTER_TO_INT(client_data) - (MAX_NUM_NATIONS * race);
 
   races_leader_set_values(race, lead);
 }
@@ -2282,8 +2317,8 @@ int races_indirect_compare(const void *first, const void *second)
 {
   int first_index;
   int second_index;
-  char *first_string;
-  char *second_string;
+  const char *first_string;
+  const char *second_string;
 
   first_index = *((const int *)first);
   second_index = *((const int *)second);
@@ -2334,8 +2369,6 @@ void races_ok_command_callback(Widget w, XtPointer client_data,
     return;
   }
 
-  packet.name[0]=toupper(packet.name[0]);
-
   send_packet_alloc_nation(&aconnection, &packet);
 }
 
@@ -2355,7 +2388,7 @@ void races_disconnect_command_callback(Widget w, XtPointer client_data,
 void races_quit_command_callback(Widget w, XtPointer client_data, 
 				 XtPointer call_data)
 {
-  exit(0);
+  exit(EXIT_SUCCESS);
 }
 
 /**************************************************************************
