@@ -36,8 +36,6 @@ enum MenuID {
   MENU_GAME_OPTIONS,
   MENU_GAME_MSG_OPTIONS,
   MENU_GAME_SAVE_SETTINGS,
-  MENU_GAME_PLAYERS,
-  MENU_GAME_MESSAGES,
   MENU_GAME_SERVER_OPTIONS1,
   MENU_GAME_SERVER_OPTIONS2,
   MENU_GAME_OUTPUT_LOG,
@@ -53,6 +51,18 @@ enum MenuID {
   MENU_VIEW_SHOW_MAP_GRID,
   MENU_VIEW_SHOW_CITY_NAMES,
   MENU_VIEW_SHOW_CITY_PRODUCTIONS,
+  MENU_VIEW_SHOW_TERRAIN,
+  MENU_VIEW_SHOW_COASTLINE,
+  MENU_VIEW_SHOW_ROADS_RAILS,
+  MENU_VIEW_SHOW_IRRIGATION,
+  MENU_VIEW_SHOW_MINES,
+  MENU_VIEW_SHOW_FORTRESS_AIRBASE,
+  MENU_VIEW_SHOW_SPECIALS,
+  MENU_VIEW_SHOW_POLLUTION,
+  MENU_VIEW_SHOW_CITIES,
+  MENU_VIEW_SHOW_UNITS,
+  MENU_VIEW_SHOW_FOCUS_UNIT,
+  MENU_VIEW_SHOW_FOG_OF_WAR,
   MENU_VIEW_CENTER_VIEW,
 
   MENU_ORDER_BUILD_CITY,
@@ -74,21 +84,25 @@ enum MenuID {
   MENU_ORDER_AUTO_ATTACK,
   MENU_ORDER_AUTO_EXPLORE,
   MENU_ORDER_CONNECT,
+  MENU_ORDER_PATROL,
   MENU_ORDER_GOTO,
   MENU_ORDER_GOTO_CITY,
   MENU_ORDER_DISBAND,
   MENU_ORDER_BUILD_WONDER,
   MENU_ORDER_TRADEROUTE,
+  MENU_ORDER_DIPLOMAT_DLG,
   MENU_ORDER_NUKE,
   MENU_ORDER_WAIT,
   MENU_ORDER_DONE,
 
-  MENU_REPORT_CITY,
-  MENU_REPORT_MILITARY,
-  MENU_REPORT_TRADE,
+  MENU_REPORT_CITIES,
+  MENU_REPORT_UNITS,
+  MENU_REPORT_PLAYERS,
+  MENU_REPORT_ECONOMY,
   MENU_REPORT_SCIENCE,
   MENU_REPORT_WOW,
   MENU_REPORT_TOP_CITIES,
+  MENU_REPORT_MESSAGES,
   MENU_REPORT_DEMOGRAPHIC,
   MENU_REPORT_SPACESHIP,
 
