@@ -44,7 +44,7 @@
 #include "mem.h"
 #include "packets.h"
 #include "worklist.h"
-
+#include "climisc.h"
 #include "clinet.h"
 
 #include "wldlg.h"
@@ -103,7 +103,7 @@ static struct worklist_report_dialog *report_dialog;
 
 static int uni_id(struct worklist *pwl, int wlinx);
 
-static void worklist_id_to_name(char buf[], int id, int is_unit, 
+static void worklist_id_to_name(char buf[], int id, bool is_unit, 
 				struct city *pcity);
 
 static void rename_worklist_callback(Widget w, XtPointer client_data, 
@@ -153,7 +153,7 @@ static void worklist_avail_help_callback(Widget w, XtPointer client_data,
 					  XtPointer call_data);
 static void worklist_show_advanced_callback(Widget w, XtPointer client_data, 
 					  XtPointer call_data);
-static void worklist_help(int id, int is_unit);
+static void worklist_help(int id, bool is_unit);
 static void worklist_populate_worklist(struct worklist_dialog *pdialog);
 static void worklist_populate_targets(struct worklist_dialog *pdialog);
 
@@ -579,7 +579,7 @@ int uni_id(struct worklist *pwl, int inx)
 /****************************************************************
 
 *****************************************************************/
-void worklist_id_to_name(char buf[], int id, int is_unit, 
+void worklist_id_to_name(char buf[], int id, bool is_unit, 
 			 struct city *pcity)
 {
   if (is_unit)
@@ -629,17 +629,15 @@ void rename_worklist_sub_callback(Widget w, XtPointer client_data,
 				  XtPointer call_data)
 {
   struct worklist_report_dialog *pdialog;
-  struct packet_player_request packet;
 
   pdialog = (struct worklist_report_dialog *)client_data;
 
   if (pdialog) {
-    packet.wl_idx = pdialog->wl_idx;
-    copy_worklist(&packet.worklist, 
-		  &pdialog->pplr->worklists[pdialog->wl_idx]);
-    strncpy(packet.worklist.name, input_dialog_get_input(w), MAX_LEN_NAME);
-    packet.worklist.name[MAX_LEN_NAME-1] = '\0';
-    send_packet_player_request(&aconnection, &packet, PACKET_PLAYER_WORKLIST);
+    strncpy(pdialog->pplr->worklists[pdialog->wl_idx].name,
+            input_dialog_get_input(w), MAX_LEN_NAME);
+    pdialog->pplr->worklists[pdialog->wl_idx].name[MAX_LEN_NAME - 1] = '\0';
+
+    update_worklist_report_dialog();
   }
   
   input_dialog_destroy(w);
@@ -652,7 +650,6 @@ void insert_worklist_callback(Widget w, XtPointer client_data,
 			      XtPointer call_data)
 {
   struct worklist_report_dialog *pdialog;
-  struct packet_player_request packet;
   int j;
 
   pdialog = (struct worklist_report_dialog *)client_data;
@@ -668,12 +665,11 @@ void insert_worklist_callback(Widget w, XtPointer client_data,
     return;
 
   /* Validate this slot. */
-  init_worklist(&packet.worklist);
-  packet.worklist.is_valid = 1;
-  strcpy(packet.worklist.name, _("empty worklist"));
-  packet.wl_idx = j;
+  init_worklist(&pdialog->pplr->worklists[j]);
+  pdialog->pplr->worklists[j].is_valid = TRUE;
+  strcpy(pdialog->pplr->worklists[j].name, _("empty worklist"));
 
-  send_packet_player_request(&aconnection, &packet, PACKET_PLAYER_WORKLIST);
+  update_worklist_report_dialog();
 }
 
 /****************************************************************
@@ -684,7 +680,6 @@ void delete_worklist_callback(Widget w, XtPointer client_data,
 			      XtPointer call_data)
 {
   struct worklist_report_dialog *pdialog;
-  struct packet_player_request packet;
   XawListReturnStruct *retList;
   int i, j;
 
@@ -700,18 +695,16 @@ void delete_worklist_callback(Widget w, XtPointer client_data,
       break;
 
   for (j = retList->list_index; j < i-1; j++) {
-    copy_worklist(&packet.worklist, &pdialog->pplr->worklists[j+1]);
-    packet.wl_idx = j;
-
-    send_packet_player_request(&aconnection, &packet, PACKET_PLAYER_WORKLIST);
+    copy_worklist(&pdialog->pplr->worklists[j], 
+                  &pdialog->pplr->worklists[j+1]);
   }
 
   /* The last worklist in the set is no longer valid -- it's been slid up
-     one slot. */
-  packet.worklist.name[0] = '\0';
-  packet.worklist.is_valid = 0;
-  packet.wl_idx = i-1;
-  send_packet_player_request(&aconnection, &packet, PACKET_PLAYER_WORKLIST);
+   * one slot. */
+  pdialog->pplr->worklists[i-1].is_valid = FALSE;
+  strcpy(pdialog->pplr->worklists[i-1].name, "\0");
+  
+  update_worklist_report_dialog();
 }
 
 /****************************************************************
@@ -742,17 +735,12 @@ void edit_worklist_callback(Widget w, XtPointer client_data,
 void commit_player_worklist(struct worklist *pwl, void *data)
 {
   struct worklist_report_dialog *pdialog;
-  struct packet_player_request packet;
 
   pdialog = (struct worklist_report_dialog *)data;
 
-  copy_worklist(&packet.worklist, pwl);
-  packet.wl_idx = pdialog->wl_idx;
-
-  send_packet_player_request(&aconnection, &packet, PACKET_PLAYER_WORKLIST);
+  copy_worklist(&pdialog->pplr->worklists[pdialog->wl_idx], pwl);
 }
 
-
 /****************************************************************
 
 *****************************************************************/
@@ -765,7 +753,6 @@ void close_worklistreport_callback(Widget w, XtPointer client_data,
   report_dialog = NULL;
 }
 
-
 /****************************************************************
   Fill in the worklist arrays in the pdialog.
 *****************************************************************/
@@ -837,19 +824,12 @@ void worklist_avail_callback(Widget w, XtPointer client_data,
 /****************************************************************
 
 *****************************************************************/
-void insert_into_worklist(struct worklist_dialog *pdialog, 
-			  int before, int id)
+void insert_into_worklist(struct worklist_dialog *pdialog,
+			  int before, cid cid)
 {
   int i, first_free;
-  int target, is_unit;
-
-  if (id >= B_LAST) {
-    target = id - B_LAST;
-    is_unit = 1;
-  } else {
-    target = id;
-    is_unit = 0;
-  }
+  int target = cid_id(cid);
+  bool is_unit = cid_is_unit(cid);
 
   /* If this worklist is a city worklist, double check that the city
      really can (eventually) build the target.  We've made sure that
@@ -893,7 +873,7 @@ void insert_into_worklist(struct worklist_dialog *pdialog,
   pdialog->worklist_ids[first_free] = WORKLIST_END;
   pdialog->worklist_names_ptrs[first_free] = NULL;
 
-  pdialog->worklist_ids[before] = id;
+  pdialog->worklist_ids[before] = cid;
   
   worklist_id_to_name(pdialog->worklist_names[before],
 		      target, is_unit, pdialog->pcity);
@@ -969,11 +949,11 @@ void worklist_insert_common_callback(struct worklist_dialog *pdialog,
   } else if (retAvail->list_index >= 
 	     pdialog->worklist_avail_num_improvements) {
     /* target is an improvement or wonder */
-    insert_into_worklist(pdialog, where, target+B_LAST);
+    insert_into_worklist(pdialog, where, cid_encode(TRUE, target));
     where++;
   } else {
     /* target is a unit */
-    insert_into_worklist(pdialog, where, target);
+    insert_into_worklist(pdialog, where, cid_encode(FALSE, target));
     where++;
   }
 
@@ -1171,19 +1151,16 @@ void worklist_worklist_help_callback(Widget w, XtPointer client_data,
 {
   struct worklist_dialog *pdialog;
   XawListReturnStruct *ret;
-  int id, is_unit = 0;
+  int id;
+  bool is_unit = FALSE;
 
   pdialog=(struct worklist_dialog *)client_data;
 
   ret = XawListShowCurrent(pdialog->worklist);
   if(ret->list_index!=XAW_LIST_NONE) {
-    id = pdialog->worklist_ids[ret->list_index];
-    if (id >= B_LAST) {
-      id -= B_LAST;
-      is_unit = 1;
-    } else {
-      is_unit = 0;
-    }
+    cid cid = pdialog->worklist_ids[ret->list_index];
+    is_unit = cid_is_unit(cid);
+    id = cid_id(cid);
   } else {
     id = -1;
   }
@@ -1196,7 +1173,8 @@ void worklist_avail_help_callback(Widget w, XtPointer client_data,
 {
   struct worklist_dialog *pdialog;
   XawListReturnStruct *ret;
-  int id, is_unit = 0;
+  int id;
+  bool is_unit = FALSE;
 
   pdialog=(struct worklist_dialog *)client_data;
 
@@ -1217,7 +1195,7 @@ void worklist_avail_help_callback(Widget w, XtPointer client_data,
 }
 
 
-void worklist_help(int id, int is_unit)
+void worklist_help(int id, bool is_unit)
 {
   if(id >= 0) {
     if (is_unit) {
@@ -1262,7 +1240,8 @@ void worklist_populate_worklist(struct worklist_dialog *pdialog)
 {
   int i, n;
   int id;
-  int target, is_unit;
+  int target;
+  bool is_unit;
 
   n = 0;
   if (pdialog->pcity) {
@@ -1321,7 +1300,7 @@ void worklist_populate_targets(struct worklist_dialog *pdialog)
     advanced_tech = False;
  
   /*     + First, improvements and Wonders. */
-  for(i=0; i<game.num_impr_types; i++) {
+  impr_type_iterate(i) {
     /* Can the player (eventually) build this improvement? */
     can_build = can_player_build_improvement(pplr,i);
     can_eventually_build = could_player_eventually_build_improvement(pplr,i);
@@ -1340,11 +1319,11 @@ void worklist_populate_targets(struct worklist_dialog *pdialog)
       pdialog->worklist_avail_names_ptrs[n]=pdialog->worklist_avail_names[n];
       pdialog->worklist_avail_ids[n++]=i;
     }
-  }
+  } impr_type_iterate_end;
   pdialog->worklist_avail_num_improvements=n;
 
   /*     + Second, units. */
-  for(i=0; i<game.num_unit_types; i++) {
+  unit_type_iterate(i) {
     /* Can the player (eventually) build this improvement? */
     can_build = can_player_build_unit(pplr,i);
     can_eventually_build = can_player_eventually_build_unit(pplr,i);
@@ -1363,7 +1342,8 @@ void worklist_populate_targets(struct worklist_dialog *pdialog)
       pdialog->worklist_avail_names_ptrs[n]=pdialog->worklist_avail_names[n];
       pdialog->worklist_avail_ids[n++]=i;
     }
-  }
+  } unit_type_iterate_end;
+
   pdialog->worklist_avail_num_targets=n;
 
   /*     + Finally, the global worklists. */
