@@ -41,7 +41,7 @@
 
 static const char * const help_type_names[] = {
   "(Any)", "(Text)", "Units", "Improvements", "Wonders",
-  "Techs", "Terrain", "Governments", 0
+  "Techs", "Terrain", "Governments", NULL
 };
 
 
@@ -59,7 +59,7 @@ static const char * const help_type_names[] = {
 
 static struct genlist help_nodes;
 static struct genlist_iterator help_nodes_iterator;
-static int help_nodes_init = 0;
+static bool help_nodes_init = FALSE;
 /* helpnodes_init is not quite the same as booted in boot_help_texts();
    latter can be 0 even after call, eg if couldn't find helpdata.txt.
 */
@@ -76,7 +76,7 @@ static void check_help_nodes_init(void)
 {
   if (!help_nodes_init) {
     genlist_init(&help_nodes);
-    help_nodes_init = 1;    /* before help_iter_start to avoid recursion! */
+    help_nodes_init = TRUE;    /* before help_iter_start to avoid recursion! */
     help_iter_start();
   }
 }
@@ -84,7 +84,7 @@ static void check_help_nodes_init(void)
 /****************************************************************
   Free all allocations associated with help_nodes.
 *****************************************************************/
-static void free_help_nodes(void)
+void free_help_texts(void)
 {
   check_help_nodes_init();
   help_list_iterate(help_nodes, ptmp) {
@@ -110,7 +110,7 @@ static void insert_generated_table(const char* name, char* outbuf)
       strcat (outbuf, "---------------------------------------------------------------\n");
       for (i = T_FIRST; i < T_COUNT; i++)
 	{
-	  if (*(tile_types[i].terrain_name))
+	  if (*(tile_types[i].terrain_name) != '\0')
 	    {
 	      outbuf = strchr (outbuf, '\0');
 	      sprintf(outbuf,
@@ -162,7 +162,7 @@ static int help_item_compar(const void *a, const void *b)
   char *ta, *tb;
   ha = (const struct help_item*) *(const void**)a;
   hb = (const struct help_item*) *(const void**)b;
-  for (ta = ha->topic, tb = hb->topic; *ta && *tb; ta++, tb++) {
+  for (ta = ha->topic, tb = hb->topic; *ta != '\0' && *tb != '\0'; ta++, tb++) {
     if (*ta != ' ') {
       if (*tb == ' ') return -1;
       break;
@@ -179,7 +179,7 @@ static int help_item_compar(const void *a, const void *b)
 *****************************************************************/
 void boot_help_texts(void)
 {
-  static int booted = 0;
+  static bool booted = FALSE;
 
   struct section_file file, *sf = &file;
   char *filename;
@@ -189,24 +189,20 @@ void boot_help_texts(void)
   int nsec, npara;
 
   check_help_nodes_init();
-  
+
   /* need to do something like this or bad things happen */
   popdown_help_dialog();
-  
+
   if(!booted) {
     freelog(LOG_VERBOSE, "Booting help texts");
   } else {
     /* free memory allocated last time booted */
-    free_help_nodes();
+    free_help_texts();
     freelog(LOG_VERBOSE, "Rebooting help texts");
   }    
-  
+
   filename = datafilename("helpdata.txt");
-  if (filename == NULL) {
-    freelog(LOG_ERROR, "Could not find readable helpdata.txt in data path");
-    freelog(LOG_ERROR, "The data path may be set via"
-	                " the environment variable FREECIV_PATH");
-    freelog(LOG_ERROR, "Current data path is: \"%s\"", datafilename(NULL));
+  if (!filename) {
     freelog(LOG_ERROR, "Did not read help texts");
     return;
   }
@@ -218,7 +214,7 @@ void boot_help_texts(void)
   }
 
   sec = secfile_get_secnames_prefix(sf, "help_", &nsec);
-  
+
   for(isec=0; isec<nsec; isec++) {
     char *gen_str =
       secfile_lookup_str_default(sf, NULL, "%s.generate", sec[isec]);
@@ -248,7 +244,7 @@ void boot_help_texts(void)
 	
 	genlist_init(&category_nodes);
 	if(current_type==HELP_UNIT) {
-	  for(i=0; i<game.num_unit_types; i++) {
+	  unit_type_iterate(i) {
 	    if(unit_type_exists(i)) {
 	      pitem = new_help_item(current_type);
 	      my_snprintf(name, sizeof(name), " %s", unit_name(i));
@@ -256,7 +252,7 @@ void boot_help_texts(void)
 	      pitem->text = mystrdup("");
 	      genlist_insert(&category_nodes, pitem, -1);
 	    }
-	  }
+	  } unit_type_iterate_end;
 	} else if(current_type==HELP_TECH) {
 	  for(i=A_FIRST; i<game.num_tech_types; i++) {  /* skip A_NONE */
 	    if(tech_exists(i)) {
@@ -269,7 +265,7 @@ void boot_help_texts(void)
 	  }
 	} else if(current_type==HELP_TERRAIN) {
 	  for(i=T_FIRST; i<T_COUNT; i++) {
-	    if(*(tile_types[i].terrain_name)) {
+	    if(*(tile_types[i].terrain_name) != '\0') {
 	      pitem = new_help_item(current_type);
 	      my_snprintf(name, sizeof(name), " %s", tile_types[i].terrain_name);
 	      pitem->topic = mystrdup(name);
@@ -295,7 +291,7 @@ void boot_help_texts(void)
 	      genlist_insert(&category_nodes, pitem, -1);
 	  }
 	} else if(current_type==HELP_IMPROVEMENT) {
-	  for(i=0; i<game.num_impr_types; i++) {
+	  impr_type_iterate(i) {
 	    if(improvement_exists(i) && !is_wonder(i)) {
 	      pitem = new_help_item(current_type);
 	      my_snprintf(name, sizeof(name), " %s", improvement_types[i].name);
@@ -303,9 +299,9 @@ void boot_help_texts(void)
 	      pitem->text = mystrdup("");
 	      genlist_insert(&category_nodes, pitem, -1);
 	    }
-	  }
+	  } impr_type_iterate_end;
 	} else if(current_type==HELP_WONDER) {
-	  for(i=0; i<game.num_impr_types; i++) {
+	  impr_type_iterate(i) {
 	    if(improvement_exists(i) && is_wonder(i)) {
 	      pitem = new_help_item(current_type);
 	      my_snprintf(name, sizeof(name), " %s", improvement_types[i].name);
@@ -313,10 +309,10 @@ void boot_help_texts(void)
 	      pitem->text = mystrdup("");
 	      genlist_insert(&category_nodes, pitem, -1);
 	    }
-	  }
+	  } impr_type_iterate_end;
 	} else {
 	  freelog(LOG_FATAL, "Bad current_type %d", current_type);
-	  exit(1);
+	  exit(EXIT_FAILURE);
 	}
 	genlist_sort(&category_nodes, help_item_compar);
 	help_list_iterate(category_nodes, ptmp) {
@@ -348,15 +344,17 @@ void boot_help_texts(void)
       }
     }
     free(paras);
+    paras = NULL;
     wordwrap_string(long_buffer, 68);
     pitem->text=mystrdup(long_buffer);
     genlist_insert(&help_nodes, pitem, -1);
   }
 
   free(sec);
+  sec = NULL;
   section_file_check_unused(sf, sf->filename);
   section_file_free(sf);
-  booted = 1;
+  booted = TRUE;
   freelog(LOG_VERBOSE, "Booted help texts ok");
 }
 
@@ -592,121 +590,175 @@ void helptext_unit(char *buf, int i, const char *user_text)
   
   buf[0] = '\0';
   if (utype->transport_capacity>0) {
-    if (unit_flag(i, F_CARRIER)) {
-      sprintf(buf+strlen(buf), _("* Can carry and refuel %d air units.\n"),
-	      utype->transport_capacity);
-    } else if (unit_flag(i, F_MISSILE_CARRIER)) {
-      sprintf(buf+strlen(buf), _("* Can carry and refuel %d missile units.\n"),
-	      utype->transport_capacity);
+    if (unit_type_flag(i, F_CARRIER)) {
+      sprintf(buf + strlen(buf),
+	      PL_("* Can carry and refuel %d air unit.\n",
+		  "* Can carry and refuel %d air units.\n",
+		  utype->transport_capacity), utype->transport_capacity);
+    } else if (unit_type_flag(i, F_MISSILE_CARRIER)) {
+      sprintf(buf + strlen(buf),
+	      PL_("* Can carry and refuel %d missile unit.\n",
+		  "* Can carry and refuel %d missile units.\n",
+		  utype->transport_capacity), utype->transport_capacity);
     } else {
-      sprintf(buf+strlen(buf), _("* Can carry %d ground units across water.\n"),
-	      utype->transport_capacity);
+      sprintf(buf + strlen(buf),
+	      PL_("* Can carry %d ground unit across water.\n",
+		  "* Can carry %d ground units across water.\n",
+		  utype->transport_capacity), utype->transport_capacity);
     }
   }
-  if (unit_flag(i, F_CARAVAN)) {
-    sprintf(buf+strlen(buf),
-	    _("* Can establish trade routes and help build wonders.\n"));
+  if (unit_type_flag(i, F_TRADE_ROUTE)) {
+    sprintf(buf + strlen(buf), _("* Can establish trade routes.\n"));
   }
-  if (unit_flag(i, F_CITIES)) {
+  if (unit_type_flag(i, F_HELP_WONDER)) {
+    sprintf(buf + strlen(buf), _("* Can help build wonders.\n"));
+  }
+  if (unit_type_flag(i, F_CITIES)) {
     sprintf(buf+strlen(buf), _("* Can build new cities.\n"));
   }
-  if (unit_flag(i, F_SETTLERS)) {
+  if (unit_type_flag(i, F_SETTLERS)) {
     sprintf(buf+strlen(buf), _("* Can perform settler actions.\n"));
   }
-  if (unit_flag(i, F_DIPLOMAT)) {
-    if (unit_flag(i, F_SPY)) 
+  if (unit_type_flag(i, F_DIPLOMAT)) {
+    if (unit_type_flag(i, F_SPY)) 
       sprintf(buf+strlen(buf), _("* Can perform diplomatic actions,"
 				 " plus special spy abilities.\n"));
     else 
       sprintf(buf+strlen(buf), _("* Can perform diplomatic actions.\n"));
   }
-  if (unit_flag(i, F_FIGHTER)) {
+  if (unit_type_flag(i, F_FIGHTER)) {
     sprintf(buf+strlen(buf), _("* Can attack enemy air units.\n"));
   }
-  if (unit_flag(i, F_PARTIAL_INVIS)) {
+  if (unit_type_flag(i, F_PARTIAL_INVIS)) {
     sprintf(buf+strlen(buf), _("* Is invisible except when next to an"
 			       " enemy unit or city.\n"));
   }
-  if (unit_flag(i, F_NO_LAND_ATTACK)) {
+  if (unit_type_flag(i, F_NO_LAND_ATTACK)) {
     sprintf(buf+strlen(buf), _("* Can only attack units on ocean squares"
 			       " (no land attacks).\n"));
   }
-  if (unit_flag(i, F_MARINES)) {
+  if (unit_type_flag(i, F_MARINES)) {
     sprintf(buf+strlen(buf), _("* Can attack from aboard sea units: against"
 			       " enemy cities and onto land squares."));
   }
-  if (unit_flag(i, F_PARATROOPERS)) {
+  if (unit_type_flag(i, F_PARATROOPERS)) {
     sprintf(buf+strlen(buf), _("* Can be paradropped from a friendly city"
 			       " (Range: %d)."), utype->paratroopers_range);
   }
-  if (unit_flag(i, F_PIKEMEN)) {
+  if (unit_type_flag(i, F_PIKEMEN)) {
     sprintf(buf+strlen(buf), _("* Gets double defense against units"
 			       " specified as 'mounted'.\n"));
   }
-  if (unit_flag(i, F_HORSE)) {
+  if (unit_type_flag(i, F_HORSE)) {
     sprintf(buf+strlen(buf),
 	    _("* Counts as 'mounted' against certain defenders.\n"));
   }
-  if (unit_flag(i, F_MISSILE)) {
+  if (unit_type_flag(i, F_MISSILE)) {
     sprintf(buf+strlen(buf),
 	    _("* A missile unit: gets used up in making an attack.\n"));
-  } else if(unit_flag(i, F_ONEATTACK)) {
+  } else if(unit_type_flag(i, F_ONEATTACK)) {
     sprintf(buf+strlen(buf), _("* Making an attack ends this unit's turn.\n"));
   }
-  if (unit_flag(i, F_NUCLEAR)) {
+  if (unit_type_flag(i, F_NUCLEAR)) {
     sprintf(buf+strlen(buf),
 	    _("* This unit's attack causes a nuclear explosion!\n"));
   }
-  if (unit_flag(i, F_IGWALL)) {
+  if (unit_type_flag(i, F_IGWALL)) {
     sprintf(buf+strlen(buf), _("* Ignores the effects of city walls.\n"));
   }
-  if (unit_flag(i, F_AEGIS)) {
+  if (unit_type_flag(i, F_AEGIS)) {
     sprintf(buf+strlen(buf),
 	    _("* Gets quintuple defence against missiles and aircraft.\n"));
   }
-  if (unit_flag(i, F_IGTER)) {
+  if (unit_type_flag(i, F_IGTER)) {
     sprintf(buf+strlen(buf),
 	    _("* Ignores terrain effects (treats all squares as roads).\n"));
   }
-  if (unit_flag(i, F_IGTIRED)) {
+  if (unit_type_flag(i, F_IGTIRED)) {
     sprintf(buf+strlen(buf),
 	    _("* Attacks with full strength even if less than one movement left.\n"));
   }
-  if (unit_flag(i, F_IGZOC)) {
+  if (unit_type_flag(i, F_IGZOC)) {
     sprintf(buf+strlen(buf), _("* Ignores zones of control.\n"));
   }
-  if (unit_flag(i, F_NONMIL)) {
+  if (unit_type_flag(i, F_NONMIL)) {
     sprintf(buf+strlen(buf), _("* A non-military unit"
 			       " (cannot attack; no martial law).\n"));
   }
-  if (unit_flag(i, F_FIELDUNIT)) {
+  if (unit_type_flag(i, F_FIELDUNIT)) {
     sprintf(buf+strlen(buf), _("* A field unit: one unhappiness applies"
 			       " even when non-aggressive.\n"));
   }
-  if (unit_flag(i, F_TRIREME)) {
+  if (unit_type_flag(i, F_TRIREME)) {
     sprintf(buf+strlen(buf),
 	    _("* Must end turn in a city or next to land,"
 	      " or has a 50%% risk of being lost at sea."));
   }
-  if (utype->fuel>0) {
-    sprintf(buf+strlen(buf), _("* Must end "));
-    if (utype->fuel==2) {
-      sprintf(buf+strlen(buf), _("second "));
-    } else if (utype->fuel==3) {
-      sprintf(buf+strlen(buf), _("third "));
-    } else if (utype->fuel>=4) {
-      sprintf(buf+strlen(buf), _("%dth "), utype->fuel);
+  if (utype->fuel > 0) {
+    char allowed_units[10][64];
+    int num_allowed_units = 0;
+    int j, n;
+    struct astring astr;
+
+    astr_init(&astr);
+    astr_minsize(&astr,1);
+    astr.str[0] = '\0';
+
+    n = num_role_units(F_CARRIER);
+    for (j = 0; j < n; j++) {
+      Unit_Type_id id = get_role_unit(F_CARRIER, j);
+
+      mystrlcpy(allowed_units[num_allowed_units],
+		unit_name(id), sizeof(allowed_units[num_allowed_units]));
+      num_allowed_units++;
+      assert(num_allowed_units < ARRAY_SIZE(allowed_units));
     }
-    /* FIXME: should use something like get_units_with_flag_string() */
-    sprintf(buf+strlen(buf), _("turn in a city, or on a Carrier"));
-    if (unit_flag(i, F_MISSILE) &&
-	num_role_units(F_MISSILE_CARRIER)>0 &&
-	get_unit_type(get_role_unit(F_MISSILE_CARRIER,0))->transport_capacity) {
-      sprintf(buf+strlen(buf), _(" or Submarine"));
+
+    if (unit_type_flag(i, F_MISSILE)) {
+      n = num_role_units(F_MISSILE_CARRIER);
+
+      for (j = 0; j < n; j++) {
+	Unit_Type_id id = get_role_unit(F_MISSILE_CARRIER, j);
+
+	if (get_unit_type(id)->transport_capacity > 0) {
+	  mystrlcpy(allowed_units[num_allowed_units],
+		    unit_name(id), sizeof(allowed_units[num_allowed_units]));
+	  num_allowed_units++;
+	  assert(num_allowed_units < ARRAY_SIZE(allowed_units));
+	}
+      }
     }
-    sprintf(buf+strlen(buf), _(", or will run out of fuel and be lost.\n"));
+
+    for (j = 0; j < num_allowed_units; j++) {
+      char *deli_str = NULL;
+
+      /* there should be something like astr_append() */
+      astr_minsize(&astr, astr.n + strlen(allowed_units[j]));
+      strcat(astr.str, allowed_units[j]);
+
+      if (j == num_allowed_units - 2) {
+	deli_str = _(" or ");
+      } else if (j < num_allowed_units - 1) {
+	deli_str = Q_("?or:, ");
+      }
+
+      if (deli_str) {
+	astr_minsize(&astr, astr.n + strlen(deli_str));
+	strcat(astr.str, deli_str);
+      }
+    }
+    
+    assert(num_allowed_units > 0);
+
+    sprintf(buf + strlen(buf),
+	    PL_("* Unit has to be in a city, or on a %s"
+		" after %d turn.\n",
+		"* Unit has to be in a city, or on a %s"
+		" after %d turns.\n", utype->fuel),
+	    astr.str, utype->fuel);
+    astr_free(&astr);
   }
-  if (strlen(buf)) {
+  if (strlen(buf) > 0) {
     sprintf(buf+strlen(buf), "\n");
   } 
   if (utype->helptext) {
@@ -726,6 +778,30 @@ void helptext_tech(char *buf, int i, const char *user_text)
   assert(buf&&user_text);
   strcpy(buf, user_text);
 
+  if (get_invention(game.player_ptr, i) != TECH_KNOWN) {
+    if (get_invention(game.player_ptr, i) == TECH_REACHABLE) {
+      sprintf(buf + strlen(buf),
+	      _("If we would now start with %s we would need %d bulbs."),
+	      advances[i].name,
+	      base_total_bulbs_required(game.player_ptr, i));
+    } else {
+      sprintf(buf + strlen(buf),
+	      _("To reach %s we need to obtain %d other "
+		"technologies first. The whole project "
+		"will require %d bulbs to complete."),
+	      advances[i].name,
+	      num_unknown_techs_for_goal(game.player_ptr, i) - 1,
+	      total_bulbs_required_for_goal(game.player_ptr, i));
+    }
+    if (!techs_have_fixed_costs()) {
+      sprintf(buf + strlen(buf),
+	      _(" This number may vary depending on what "
+		"other players will research.\n"));
+    } else {
+      sprintf(buf + strlen(buf), "\n");
+    }
+  }
+
   for(gov=0; gov<game.government_count; gov++) {
     struct government *g = get_government(gov);
     if (g->required_tech == i) {
@@ -752,35 +828,35 @@ void helptext_tech(char *buf, int i, const char *user_text)
     sprintf(buf+strlen(buf), _("Improves the effect of Temples.\n"));
 
   if(tech_flag(i,TF_BRIDGE)) {
-    char *units_str = get_units_with_flag_string(F_SETTLERS);
+    const char *units_str = get_units_with_flag_string(F_SETTLERS);
     sprintf(buf+strlen(buf), _("Allows %s to build roads on river squares.\n"),units_str);
-    free(units_str);
+    free((void *) units_str);
   }
 
   if(tech_flag(i,TF_FORTRESS)) {
-    char *units_str = get_units_with_flag_string(F_SETTLERS);
+    const char *units_str = get_units_with_flag_string(F_SETTLERS);
     sprintf(buf+strlen(buf), _("Allows %s to build fortresses.\n"),units_str);
-    free(units_str);
+    free((void *) units_str);
   }
 
   if(tech_flag(i,TF_AIRBASE)) {
-    char *units_str = get_units_with_flag_string(F_AIRBASE);
+    const char *units_str = get_units_with_flag_string(F_AIRBASE);
     if (units_str) {
       sprintf(buf+strlen(buf), _("Allows %s to build airbases.\n"),units_str);
-      free(units_str);
+      free((void *) units_str);
     }
   }
 
   if(tech_flag(i,TF_RAILROAD)) {
-    char *units_str = get_units_with_flag_string(F_SETTLERS);
+    const char *units_str = get_units_with_flag_string(F_SETTLERS);
     sprintf(buf+strlen(buf), _("Allows %s to upgrade roads to railroads.\n"),units_str);
-    free(units_str);
+    free((void *) units_str);
   }
 
   if(tech_flag(i,TF_FARMLAND)) {
-    char *units_str = get_units_with_flag_string(F_SETTLERS);
+    const char *units_str = get_units_with_flag_string(F_SETTLERS);
     sprintf(buf+strlen(buf), _("Allows %s to upgrade irrigation to farmland.\n"),units_str);
-    free(units_str);
+    free((void *) units_str);
   }
 }
 
@@ -828,25 +904,29 @@ char *helptext_unit_upkeep_str(int i)
   static char buf[128];
   struct unit_type *utype = get_unit_type(i);
 
-  if (utype->shield_cost || utype->food_cost
-      || utype->gold_cost || utype->happy_cost) {
+  if (utype->shield_cost > 0 || utype->food_cost > 0
+      || utype->gold_cost > 0 || utype->happy_cost > 0) {
     int any = 0;
     buf[0] = '\0';
-    if (utype->shield_cost) {
+    if (utype->shield_cost > 0) {
       sprintf(buf+strlen(buf), _("%s%d shield"),
-	      (any++ ? ", " : ""), utype->shield_cost);
+	      (any > 0 ? ", " : ""), utype->shield_cost);
+      any++;
     }
-    if (utype->food_cost) {
+    if (utype->food_cost > 0) {
       sprintf(buf+strlen(buf), _("%s%d food"),
-	      (any++ ? ", " : ""), utype->food_cost);
+	      (any > 0 ? ", " : ""), utype->food_cost);
+      any++;
     }
-    if (utype->happy_cost) {
+    if (utype->happy_cost > 0) {
       sprintf(buf+strlen(buf), _("%s%d unhappy"),
-	      (any++ ? ", " : ""), utype->happy_cost);
+	      (any > 0 ? ", " : ""), utype->happy_cost);
+      any++;
     }
-    if (utype->gold_cost) {
+    if (utype->gold_cost > 0) {
       sprintf(buf+strlen(buf), _("%s%d gold"),
-	      (any++ ? ", " : ""), utype->gold_cost);
+	      (any > 0 ? ", " : ""), utype->gold_cost);
+      any++;
     }
   } else {
     /* strcpy(buf, _("None")); */
