@@ -13,9 +13,11 @@
 #ifndef FC__CITYREP_G_H
 #define FC__CITYREP_G_H
 
+#include "shared.h"		/* bool type */
+
 struct city;
 
-void popup_city_report_dialog(int make_modal);
+void popup_city_report_dialog(bool make_modal);
 void city_report_dialog_update(void);
 void city_report_dialog_update_city(struct city *pcity);
 
