@@ -13,6 +13,10 @@
 #ifndef FC__MAPCTRL_G_H
 #define FC__MAPCTRL_G_H
 
+#include "shared.h"		/* bool type */
+
+#include "mapctrl_common.h"
+
 struct unit;
 struct city;
 
@@ -20,7 +24,7 @@ extern struct city *city_workers_display;
 
 void popup_newcity_dialog(struct unit *punit, char *suggestname);
 
-void set_turn_done_button_state( int state );
+void set_turn_done_button_state(bool state);
 
 void create_line_at_mouse_pos(void);
 
