@@ -13,16 +13,19 @@
 #ifndef FC__MAPVIEW_G_H
 #define FC__MAPVIEW_G_H
 
+#include "mapview_common.h"
+#include "shared.h"		/* bool type */
+
 struct unit;
 struct city;
 
-int tile_visible_mapcanvas(int x, int y);
-int tile_visible_and_not_on_border_mapcanvas(int x, int y);
+bool tile_visible_mapcanvas(int x, int y);
+bool tile_visible_and_not_on_border_mapcanvas(int x, int y);
 
 void update_info_label(void);
 void update_unit_info_label(struct unit *punit);
 void update_timeout_label(void);
-void update_turn_done_button(int do_restore);
+void update_turn_done_button(bool do_restore);
 void update_city_descriptions(void);
 void set_indicator_icons(int bulb, int sol, int flake, int gov);
 
@@ -33,21 +36,20 @@ void center_tile_mapcanvas(int x, int y);
 void get_center_tile_mapcanvas(int *x, int *y);
 
 void update_map_canvas(int x, int y, int width, int height,
-		       int write_to_screen);
+		       bool write_to_screen);
 void update_map_canvas_visible(void);
 void update_map_canvas_scrollbars(void);
 
 void put_cross_overlay_tile(int x,int y);
 void put_city_workers(struct city *pcity, int color);
 
-void move_unit_map_canvas(struct unit *punit, int x0, int y0, int x1, int y1);
+void move_unit_map_canvas(struct unit *punit, int x0, int y0, int dx, int dy);
 void decrease_unit_hp_smooth(struct unit *punit0, int hp0, 
 			     struct unit *punit1, int hp1);
 void put_nuke_mushroom_pixmaps(int x, int y);
 
 void refresh_overview_canvas(void);
 void refresh_overview_viewrect(void);
-void refresh_tile_mapcanvas(int x, int y, int write_to_screen);
 
 void draw_segment(int src_x, int src_y, int dir);
 void undraw_segment(int src_x, int src_y, int dir);
