@@ -20,11 +20,18 @@
 #endif
 
 #include <errno.h>
+#include <signal.h>
 #include <string.h>
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_SIGNAL_H
+#include <sys/signal.h>
+#endif
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
@@ -46,6 +53,7 @@
 
 #include "log.h"
 #include "support.h"
+#include "shared.h"		/* TRUE, FALSE */
 
 #include "netintf.h"
 
@@ -102,6 +110,11 @@ void my_init_network(void)
     freelog(LOG_ERROR, "no useable WINSOCK.DLL: %s", mystrerror(errno));
   }
 #endif
+
+  /* broken pipes are ignored. */
+#ifdef HAVE_SIGPIPE
+  (void) signal(SIGPIPE, SIG_IGN);
+#endif
 }
 
 /***************************************************************
@@ -150,26 +163,26 @@ void my_nonblock(int sockfd)
   Look up the given host and fill in *sock.  Note that the caller
   should fill in the port number (sock->sin_port).
 ***************************************************************************/
-int fc_lookup_host(const char *hostname, struct sockaddr_in *sock)
+bool fc_lookup_host(const char *hostname, struct sockaddr_in *sock)
 {
   struct hostent *hp;
 
   sock->sin_family = AF_INET;
 
 #ifdef HAVE_INET_ATON
-  if (inet_aton(hostname, &sock->sin_addr)) {
-    return 1;
+  if (inet_aton(hostname, &sock->sin_addr) != 0) {
+    return TRUE;
   }
 #else
   if ((sock->sin_addr.s_addr = inet_addr(hostname)) != INADDR_NONE) {
-    return 1;
+    return TRUE;
   }
 #endif
   hp = gethostbyname(hostname);
-  if (hp == NULL || hp->h_addrtype != AF_INET) {
-    return 0;
+  if (!hp || hp->h_addrtype != AF_INET) {
+    return FALSE;
   }
 
   memcpy(&sock->sin_addr, hp->h_addr, hp->h_length);
-  return 1;
+  return TRUE;
 }
