@@ -13,7 +13,7 @@
 #ifndef FC__REGISTRY_H
 #define FC__REGISTRY_H
 
-#include "attribute.h"
+#include "shared.h"		/* bool type and fc__attribute */
 
 struct sbuffer;
 struct hash_data;
@@ -28,11 +28,11 @@ struct section_file {
 };
 
 void section_file_init(struct section_file *file);
-int section_file_load(struct section_file *my_section_file,
+bool section_file_load(struct section_file *my_section_file,
 		      const char *filename);
-int section_file_load_nodup(struct section_file *my_section_file,
+bool section_file_load_nodup(struct section_file *my_section_file,
 			    const char *filename);
-int section_file_save(struct section_file *my_section_file,
+bool section_file_save(struct section_file *my_section_file,
 		      const char *filename, int compression_level);
 void section_file_free(struct section_file *file);
 void section_file_check_unused(struct section_file *file,
@@ -42,18 +42,33 @@ const char *secfile_filename(const struct section_file *file);
 void secfile_insert_int(struct section_file *my_section_file, 
 			int val, char *path, ...)
                         fc__attribute((format (printf, 3, 4)));
-			
+void secfile_insert_int_comment(struct section_file *my_section_file,
+				int val, const char *const comment, 
+				char *path, ...)
+                                fc__attribute((format(printf, 4, 5)));
+void secfile_insert_bool(struct section_file *my_section_file,
+			 bool val, char *path, ...)
+                         fc__attribute((format(printf, 3, 4)));
+
 void secfile_insert_str(struct section_file *my_section_file, 
 			char *sval, char *path, ...)
                         fc__attribute((format (printf, 3, 4)));
+void secfile_insert_str_comment(struct section_file *my_section_file,
+				char *sval, const char *const comment,
+				char *path, ...)
+                                fc__attribute((format (printf, 4, 5)));
 
-int section_file_lookup(struct section_file *my_section_file, 
+bool section_file_lookup(struct section_file *my_section_file, 
 			char *path, ...)
                         fc__attribute((format (printf, 2, 3)));
 
 int secfile_lookup_int(struct section_file *my_section_file, 
 		       char *path, ...)
                        fc__attribute((format (printf, 2, 3)));
+
+bool secfile_lookup_bool(struct section_file *my_section_file, 
+		        char *path, ...)
+                        fc__attribute((format (printf, 2, 3)));
 		       
 char *secfile_lookup_str(struct section_file *my_section_file, 
 			 char *path, ...)
@@ -67,6 +82,10 @@ int secfile_lookup_int_default(struct section_file *my_section_file,
                                int def, char *path, ...)
                                fc__attribute((format (printf, 3, 4)));
 
+bool secfile_lookup_bool_default(struct section_file *my_section_file,
+				 bool def, char *path, ...)
+                                 fc__attribute((format (printf, 3, 4)));
+
 char *secfile_lookup_str_default(struct section_file *my_section_file, 
                                  char *def, char *path, ...)
                                  fc__attribute((format (printf, 3, 4)));
@@ -87,8 +106,8 @@ char **secfile_get_secnames_prefix(struct section_file *my_section_file,
 /* Only need use these explicitly on section_files constructed,
    not for ones generated by section_file_load:
 */
-int secfilehash_hashash(struct section_file *file);
-void secfilehash_build(struct section_file *file, int allow_duplicates);
+bool secfilehash_hashash(struct section_file *file);
+void secfilehash_build(struct section_file *file, bool allow_duplicates);
 void secfilehash_free(struct section_file *file);
 
 #endif  /* FC__REGISTRY_H */
