@@ -23,13 +23,45 @@ enum special_river_move {
 };
 
 enum tile_special_type {
-  S_NO_SPECIAL=0,
-  S_SPECIAL_1=1, S_ROAD=2, S_IRRIGATION=4, S_RAILROAD=8,
-  S_MINE=16, S_POLLUTION=32, S_HUT=64, S_FORTRESS=128,
-  S_SPECIAL_2=256, S_RIVER=512, S_FARMLAND=1024, S_AIRBASE=2048,
-  S_FALLOUT=4096
+  S_NO_SPECIAL =    0,
+  S_SPECIAL_1  =    1,
+  S_ROAD       =    2,
+  S_IRRIGATION =    4,
+  S_RAILROAD   =    8,
+  S_MINE       =   16,
+  S_POLLUTION  =   32,
+  S_HUT        =   64,
+  S_FORTRESS   =  128,
+  S_SPECIAL_2  =  256,
+  S_RIVER      =  512,
+  S_FARMLAND   = 1024,
+  S_AIRBASE    = 2048,
+  S_FALLOUT    = 4096
 };
-#define S_ALL (0xFFFF)
+
+#define S_ALL    \
+ (  S_SPECIAL_1  \
+  | S_ROAD       \
+  | S_IRRIGATION \
+  | S_RAILROAD   \
+  | S_MINE       \
+  | S_POLLUTION  \
+  | S_HUT        \
+  | S_FORTRESS   \
+  | S_SPECIAL_2  \
+  | S_RIVER      \
+  | S_FARMLAND   \
+  | S_AIRBASE    \
+  | S_FALLOUT)
+
+#define S_INFRASTRUCTURE_MASK \
+  (S_ROAD                   \
+   | S_RAILROAD             \
+   | S_IRRIGATION           \
+   | S_FARMLAND             \
+   | S_MINE                 \
+   | S_FORTRESS             \
+   | S_AIRBASE)
 
 enum tile_terrain_type {
   T_ARCTIC, T_DESERT, T_FOREST, T_GRASSLAND, T_HILLS, T_JUNGLE, 
@@ -43,5 +75,4 @@ enum known_type {
  TILE_UNKNOWN, TILE_KNOWN_FOGGED, TILE_KNOWN
 };
 
-
 #endif  /* FC__TERRAIN_H */
