@@ -25,10 +25,10 @@
 #include "player.h"
 #include "support.h"
 #include "tech.h"
+#include "shared.h"
 
 #include "unittype.h"
 
-
 struct unit_type unit_types[U_LAST];
 /* the unit_types array is now setup in:
    server/ruleset.c (for the server)
@@ -39,11 +39,12 @@ static const char *move_type_names[] = {
   "Land", "Sea", "Heli", "Air"
 };
 static const char *flag_names[] = {
-  "Caravan", "Missile", "IgZOC", "NonMil", "IgTer", "Carrier",
-  "OneAttack", "Pikemen", "Horse", "IgWall", "FieldUnit", "AEGIS",
-  "Fighter", "Marines", "Partial_Invis", "Settlers", "Diplomat",
+  "TradeRoute" ,"HelpWonder", "Missile", "IgZOC", "NonMil", "IgTer", 
+  "Carrier", "OneAttack", "Pikemen", "Horse", "IgWall", "FieldUnit", 
+  "AEGIS", "Fighter", "Marines", "Partial_Invis", "Settlers", "Diplomat",
   "Trireme", "Nuclear", "Spy", "Transform", "Paratroopers",
-  "Airbase", "Cities", "IgTired", "Missile_Carrier", "No_Land_Attack"
+  "Airbase", "Cities", "IgTired", "Missile_Carrier", "No_Land_Attack",
+  "AddToCity", "Fanatic"
 };
 static const char *role_names[] = {
   "FirstBuild", "Explorer", "Hut", "HutTech", "Partisan",
@@ -68,10 +69,10 @@ A unit_type doesn't exist if one of:
 - the unit_type has been flagged as removed by setting its
   tech_requirement to A_LAST.
 **************************************************************************/
-int unit_type_exists(Unit_Type_id id)
+bool unit_type_exists(Unit_Type_id id)
 {
   if (id<0 || id>=U_LAST || id>=game.num_unit_types)
-    return 0;
+    return FALSE;
   else 
     return unit_types[id].tech_requirement!=A_LAST;
 }
@@ -81,13 +82,22 @@ int unit_type_exists(Unit_Type_id id)
 **************************************************************************/
 struct unit_type *get_unit_type(Unit_Type_id id)
 {
+  assert(id >= 0 && id < U_LAST && id < game.num_unit_types);
   return &unit_types[id];
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-int is_ground_unittype(Unit_Type_id id)
+struct unit_type *unit_type(struct unit *punit)
+{
+  return get_unit_type(punit->type);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+bool is_ground_unittype(Unit_Type_id id)
 {
   return (unit_types[id].move_type == LAND_MOVING);
 }
@@ -95,7 +105,7 @@ int is_ground_unittype(Unit_Type_id id)
 /**************************************************************************
 ...
 **************************************************************************/
-int is_air_unittype(Unit_Type_id id)
+bool is_air_unittype(Unit_Type_id id)
 {
   return (unit_types[id].move_type == AIR_MOVING);
 }
@@ -103,7 +113,7 @@ int is_air_unittype(Unit_Type_id id)
 /**************************************************************************
 ...
 **************************************************************************/
-int is_heli_unittype(Unit_Type_id id)
+bool is_heli_unittype(Unit_Type_id id)
 {
   return (unit_types[id].move_type == HELI_MOVING);
 }
@@ -111,7 +121,7 @@ int is_heli_unittype(Unit_Type_id id)
 /**************************************************************************
 ...
 **************************************************************************/
-int is_water_unit(Unit_Type_id id)
+bool is_water_unit(Unit_Type_id id)
 {
   return (unit_types[id].move_type == SEA_MOVING);
 }
@@ -121,6 +131,10 @@ int is_water_unit(Unit_Type_id id)
 **************************************************************************/
 int utype_shield_cost(struct unit_type *ut, struct government *g)
 {
+  if (government_has_flag(g, G_FANATIC_TROOPS) &&
+      BV_ISSET(ut->flags, F_FANATIC)) {
+    return 0;
+  }
   return ut->shield_cost * g->unit_shield_cost_factor;
 }
 
@@ -151,19 +165,27 @@ int utype_gold_cost(struct unit_type *ut, struct government *g)
 /**************************************************************************
 ...
 **************************************************************************/
-int unit_flag(Unit_Type_id id, int flag)
+bool unit_type_flag(Unit_Type_id id, int flag)
 {
   assert(flag>=0 && flag<F_LAST);
-  return BOOL_VAL(unit_types[id].flags & (1<<flag));
+  return BV_ISSET(unit_types[id].flags, flag);
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-int unit_has_role(Unit_Type_id id, int role)
+bool unit_flag(struct unit *punit, enum unit_flag_id flag)
+{
+  return unit_type_flag(punit->type, flag);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+bool unit_has_role(Unit_Type_id id, int role)
 {
   assert(role>=L_FIRST && role<L_LAST);
-  return BOOL_VAL(unit_types[id].roles & (1<<(role-L_FIRST)));
+  return BV_ISSET(unit_types[id].roles, role - L_FIRST);
 }
 
 /**************************************************************************
@@ -177,7 +199,15 @@ int unit_value(Unit_Type_id id)
 /**************************************************************************
 ...
 **************************************************************************/
-char *unit_name(Unit_Type_id id)
+int unit_pop_value(Unit_Type_id id)
+{
+  return (unit_types[id].pop_cost);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+const char *unit_name(Unit_Type_id id)
 {
   return (unit_types[id].name);
 }
@@ -185,12 +215,12 @@ char *unit_name(Unit_Type_id id)
 /**************************************************************************
 ...
 **************************************************************************/
-char *get_unit_name(Unit_Type_id id)
+const char *get_unit_name(Unit_Type_id id)
 {
   struct unit_type *ptype;
   static char buffer[256];
   ptype =get_unit_type(id);
-  if (ptype->fuel) {
+  if (ptype->fuel > 0) {
     my_snprintf(buffer, sizeof(buffer),
 		"%s [%d/%d/%d(%d)]", ptype->name, ptype->attack_strength,
 		ptype->defense_strength,
@@ -223,23 +253,23 @@ const char *unit_class_name(Unit_Class_id id)
  TODO: if there are more than 4 units with this flag return
        a fallback string (e.g. first unit name + "and similar units"
 **************************************************************************/
-char *get_units_with_flag_string(int flag)
+const char *get_units_with_flag_string(int flag)
 {
   int count=num_role_units(flag);
 
   if(count==1)
     return mystrdup(unit_name(get_role_unit(flag,0)));
 
-  if(count) {
+  if(count > 0) {
     struct astring astr;
 
     astr_init(&astr);
     astr_minsize(&astr,1);
     astr.str[0] = 0;
 
-    while(count--) {
+    while((count--) > 0) {
       int u = get_role_unit(flag,count);
-      char *unitname = unit_name(u);
+      const char *unitname = unit_name(u);
 
       /* there should be something like astr_append() */
       astr_minsize(&astr,astr.n+strlen(unitname));
@@ -252,8 +282,10 @@ char *get_units_with_flag_string(int flag)
       }
       else {
         if(count != 0) {
-          astr_minsize(&astr,astr.n+strlen(", "));
-          strcat(astr.str,", ");
+	  char *and_comma = Q_("?and:, ");
+
+	  astr_minsize(&astr, astr.n + strlen(and_comma));
+	  strcat(astr.str, and_comma);
         }
         else return astr.str;
       }
@@ -296,14 +328,13 @@ int unit_upgrade_price(struct player *pplayer, Unit_Type_id from,
 Does a linear search of unit_types[].name
 Returns U_LAST if none match.
 **************************************************************************/
-Unit_Type_id find_unit_type_by_name(char *s)
+Unit_Type_id find_unit_type_by_name(const char *s)
 {
-  int i;
-
-  for( i=0; i<game.num_unit_types; i++ ) {
+  unit_type_iterate(i) {
     if (strcmp(unit_types[i].name, s)==0)
       return i;
-  }
+  } unit_type_iterate_end;
+
   return U_LAST;
 }
 
@@ -311,12 +342,12 @@ Unit_Type_id find_unit_type_by_name(char *s)
   Convert unit_move_type names to enum; case insensitive;
   returns 0 if can't match.
 **************************************************************************/
-enum unit_move_type unit_move_type_from_str(char *s)
+enum unit_move_type unit_move_type_from_str(const char *s)
 {
   enum unit_move_type i;
 
   /* a compile-time check would be nicer, but this will do: */
-  assert(sizeof(move_type_names)/sizeof(char*)==AIR_MOVING-LAND_MOVING+1);
+  assert(ARRAY_SIZE(move_type_names) == (AIR_MOVING - LAND_MOVING + 1));
 
   for(i=LAND_MOVING; i<=AIR_MOVING; i++) {
     if (mystrcasecmp(move_type_names[i-LAND_MOVING], s)==0) {
@@ -330,11 +361,11 @@ enum unit_move_type unit_move_type_from_str(char *s)
   Convert Unit_Class_id names to enum; case insensitive;
   returns UCL_LAST if can't match.
 **************************************************************************/
-Unit_Class_id unit_class_from_str(char *s)
+Unit_Class_id unit_class_from_str(const char *s)
 {
   Unit_Class_id i;
 
-  assert(sizeof(unit_class_names)/sizeof(unit_class_names[0])==UCL_LAST);
+  assert(ARRAY_SIZE(unit_class_names) == UCL_LAST);
 
   for (i = 0; i < UCL_LAST; i++) {
     if (mystrcasecmp(unit_class_names[i], s)==0) {
@@ -348,11 +379,11 @@ Unit_Class_id unit_class_from_str(char *s)
   Convert flag names to enum; case insensitive;
   returns F_LAST if can't match.
 **************************************************************************/
-enum unit_flag_id unit_flag_from_str(char *s)
+enum unit_flag_id unit_flag_from_str(const char *s)
 {
   enum unit_flag_id i;
 
-  assert(sizeof(flag_names)/sizeof(char*)==F_LAST);
+  assert(ARRAY_SIZE(flag_names) == F_LAST);
   
   for(i=0; i<F_LAST; i++) {
     if (mystrcasecmp(flag_names[i], s)==0) {
@@ -366,11 +397,11 @@ enum unit_flag_id unit_flag_from_str(char *s)
   Convert role names to enum; case insensitive;
   returns L_LAST if can't match.
 **************************************************************************/
-enum unit_role_id unit_role_from_str(char *s)
+enum unit_role_id unit_role_from_str(const char *s)
 {
   enum unit_role_id i;
 
-  assert(sizeof(role_names)/sizeof(char*)==L_LAST-L_FIRST);
+  assert(ARRAY_SIZE(role_names) == (L_LAST - L_FIRST));
   
   for(i=L_FIRST; i<L_LAST; i++) {
     if (mystrcasecmp(role_names[i-L_FIRST], s)==0) {
@@ -385,29 +416,32 @@ Whether player can build given unit somewhere,
 ignoring whether unit is obsolete and assuming the
 player has a coastal city.
 **************************************************************************/
-int can_player_build_unit_direct(struct player *p, Unit_Type_id id)
-{  
+bool can_player_build_unit_direct(struct player *p, Unit_Type_id id)
+{
   if (!unit_type_exists(id))
-    return 0;
-  if (unit_flag(id, F_NUCLEAR) && !game.global_wonders[B_MANHATTEN])
-    return 0;
+    return FALSE;
+  if (unit_type_flag(id, F_NUCLEAR) && game.global_wonders[B_MANHATTEN] == 0)
+    return FALSE;
+  if (unit_type_flag(id, F_FANATIC)
+      && !government_has_flag(get_gov_pplayer(p), G_FANATIC_TROOPS))
+    return FALSE;
   if (get_invention(p,unit_types[id].tech_requirement)!=TECH_KNOWN)
-    return 0;
-  return 1;
+    return FALSE;
+  return TRUE;
 }
 
 /**************************************************************************
 Whether player can build given unit somewhere;
 returns 0 if unit is obsolete.
 **************************************************************************/
-int can_player_build_unit(struct player *p, Unit_Type_id id)
+bool can_player_build_unit(struct player *p, Unit_Type_id id)
 {  
   if (!can_player_build_unit_direct(p, id))
-    return 0;
+    return FALSE;
   while(unit_type_exists((id = unit_types[id].obsoleted_by)))
     if (can_player_build_unit_direct(p, id))
-	return 0;
-  return 1;
+	return FALSE;
+  return TRUE;
 }
 
 /**************************************************************************
@@ -415,14 +449,14 @@ Whether player can _eventually_ build given unit somewhere -- ie,
 returns 1 if unit is available with current tech OR will be available
 with future tech.  returns 0 if unit is obsolete.
 **************************************************************************/
-int can_player_eventually_build_unit(struct player *p, Unit_Type_id id)
+bool can_player_eventually_build_unit(struct player *p, Unit_Type_id id)
 {
   if (!unit_type_exists(id))
-    return 0;
+    return FALSE;
   while(unit_type_exists((id = unit_types[id].obsoleted_by)))
     if (can_player_build_unit_direct(p, id))
-	return 0;
-  return 1;
+	return FALSE;
+  return TRUE;
 }
 
 /**************************************************************************
@@ -433,31 +467,32 @@ and any "role" argument can also be a "flag".
 Only units which pass unit_type_exists are counted.
 Unit order is in terms of the order in the units ruleset.
 **************************************************************************/
-static int first_init = 1;
+static bool first_init = TRUE;
 static int n_with_role[L_LAST];
 static Unit_Type_id *with_role[L_LAST];
 
 /**************************************************************************
 Do the real work for role_unit_precalcs, for one role (or flag), given by i.
 **************************************************************************/
-static void precalc_one(int i, int (*func_has)(Unit_Type_id, int))
+static void precalc_one(int i, bool (*func_has)(Unit_Type_id, int))
 {
-  Unit_Type_id u;
   int j;
 
   /* Count: */
-  for(u=0; u<game.num_unit_types; u++) {
+  unit_type_iterate(u) {
     if(unit_type_exists(u) && func_has(u, i)) {
       n_with_role[i]++;
     }
-  }
+  } unit_type_iterate_end;
+
   if(n_with_role[i] > 0) {
     with_role[i] = fc_malloc(n_with_role[i]*sizeof(Unit_Type_id));
-    for(j=0, u=0; u<game.num_unit_types; u++) {
+    j = 0;
+    unit_type_iterate(u) {
       if(unit_type_exists(u) && func_has(u, i)) {
 	with_role[i][j++] = u;
       }
-    }
+    } unit_type_iterate_end;
     assert(j==n_with_role[i]);
   }
 }
@@ -481,12 +516,12 @@ void role_unit_precalcs(void)
   }
 
   for(i=0; i<F_LAST; i++) {
-    precalc_one(i, unit_flag);
+    precalc_one(i, unit_type_flag);
   }
   for(i=L_FIRST; i<L_LAST; i++) {
     precalc_one(i, unit_has_role);
   }
-  first_init = 0;
+  first_init = FALSE;
 }
 
 /**************************************************************************
@@ -512,7 +547,7 @@ Unit_Type_id get_role_unit(int role, int index)
 
 /**************************************************************************
 Return "best" unit this city can build, with given role/flag.
-Returns U_LAST if none match.
+Returns U_LAST if none match. "Best" means highest unit type id.
 **************************************************************************/
 Unit_Type_id best_role_unit(struct city *pcity, int role)
 {
@@ -530,3 +565,23 @@ Unit_Type_id best_role_unit(struct city *pcity, int role)
   return U_LAST;
 }
 
+/**************************************************************************
+  Frees the memory associated with this unit type.
+**************************************************************************/
+void unit_type_free(Unit_Type_id id)
+{
+  struct unit_type *p = get_unit_type(id);
+
+  free(p->helptext);
+  p->helptext = NULL;
+}
+
+/***************************************************************
+ Frees the memory associated with all unit types.
+***************************************************************/
+void unit_types_free(void)
+{
+  unit_type_iterate(i) {
+    unit_type_free(i);
+  } unit_type_iterate_end;
+}
