@@ -26,11 +26,16 @@
 
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 
 #include "combat.h"
 #include "events.h"
+#include "fcintl.h"
 #include "game.h"
 #include "log.h"
 #include "map.h"
@@ -62,7 +67,7 @@ static struct unit *search_best_target(struct player *pplayer,
      range is larger, because they are not going to return this turn anyways
   */
 
-  range = unit_flag(punit->type,F_ONEATTACK) ?
+  range = unit_flag(punit, F_ONEATTACK) ?
       punit->moves_left :
       punit->moves_left/2;
 
@@ -75,7 +80,7 @@ static struct unit *search_best_target(struct player *pplayer,
 	  range, punit->moves_left, pcity->city_options);
   
   square_iterate(punit->x, punit->y, range, x, y) {
-    if (x == punit->x && y == punit->y)
+    if (same_pos(punit->x, punit->y, x, y))
       continue;
 
     if (map_get_city(x, y)) continue;
@@ -83,15 +88,18 @@ static struct unit *search_best_target(struct player *pplayer,
 
     targets = &(map_get_tile(x, y)->units);
     if (unit_list_size(targets) == 0) continue;
-    if (!is_enemy_unit_tile(map_get_tile(x, y), pplayer->player_no))
+    if (!is_enemy_unit_tile(map_get_tile(x, y), pplayer))
       continue;
 
     freelog(LOG_DEBUG,  "found enemy unit/stack at %d,%d", x, y);
     enemy = get_defender(punit, x, y);
+    if (!enemy) {
+      continue;
+    }
     freelog(LOG_DEBUG,  "defender is %s", unit_name(enemy->type));
 
-    if (!(pcity->city_options
-	  & 1<<(get_unit_type(enemy->type)->move_type-1+CITYO_ATT_LAND))) {
+    if (!is_city_option_set(pcity, CITYO_ATT_LAND
+			    + unit_type(enemy)->move_type - 1)) {
       freelog(LOG_DEBUG, "wrong target type");
       continue;
     }
@@ -105,30 +113,30 @@ static struct unit *search_best_target(struct player *pplayer,
     /*
      *  perhaps there is a better algorithm in the ai-package -- fisch
      */
-    score = (get_unit_type(enemy->type)->defense_strength
-	     + (enemy->hp/2)
-	     + (get_transporter_capacity(enemy) ? 1 : 0));
+    score = (unit_type(enemy)->defense_strength + (enemy->hp / 2)
+	     + (get_transporter_capacity(enemy) > 0 ? 1 : 0));
 
-    if(best_enemy == NULL || score >= best_score) {
+    if(!best_enemy || score >= best_score) {
       best_score = score;
       best_enemy = enemy;
     }
   } square_iterate_end;
 
-  if(best_enemy == NULL) return NULL;
+  if(!best_enemy) return NULL;
 
   enemy = best_enemy;
   
   freelog(LOG_DEBUG,"choosen target=%s (%d/%d)",
 	  get_unit_name(enemy->type), enemy->x,enemy->y);
 
-  if((get_unit_type(enemy->type)->defense_strength) >
-     get_unit_type(punit->type)->attack_strength*1.5) {
+  if((unit_type(enemy)->defense_strength) >
+     unit_type(punit)->attack_strength*1.5) {
     notify_player_ex(pplayer, punit->x, punit->y, E_NOEVENT,
-		     "Game: Auto-Attack: %s's %s found a too tough enemy (%s)",
+		     _("Game: Auto-Attack: %s's %s found a too "
+		       "tough enemy (%s)"),
 		     pcity->name, unit_name(punit->type),
 		     unit_name(enemy->type));
-    punit->ai.control=0;
+    punit->ai.control=FALSE;
     handle_unit_activity_request(punit, ACTIVITY_IDLE);
     return NULL;
   }
@@ -144,27 +152,25 @@ static void auto_attack_with_unit(struct player *pplayer, struct city *pcity,
 {
   int id = punit->id;
   struct unit *enemy;
-  int debug = 0;
 
   enemy=search_best_target(pplayer,pcity,punit);
 
   /* nothing found */
-  if(enemy == NULL) return;
+  if(!enemy) return;
   
-  if (debug) freelog(LOG_DEBUG, "launching attack");
+  freelog(LOG_DEBUG, "launching attack");
   
-  notify_player_ex(pplayer, enemy->x,enemy->y, E_NOEVENT,
-                   "Game: Auto-Attack: %s's %s attacking %s's %s",
-                   pcity->name, unit_name(punit->type),
-                   get_player(enemy->owner)->name,
-                   unit_name(enemy->type));
+  notify_player_ex(pplayer, enemy->x, enemy->y, E_NOEVENT,
+		   _("Game: Auto-Attack: %s's %s attacking %s's %s"),
+		   pcity->name, unit_name(punit->type),
+		   unit_owner(enemy)->name, unit_name(enemy->type));
   
   set_unit_activity(punit, ACTIVITY_GOTO);
   punit->goto_dest_x=enemy->x;
   punit->goto_dest_y=enemy->y;
   
-  send_unit_info(0,punit);
-  do_unit_goto(punit, GOTO_MOVE_ANY, 0);
+  send_unit_info(NULL, punit);
+  (void) do_unit_goto(punit, GOTO_MOVE_ANY, FALSE);
   
   punit = find_unit_by_id(id);
   
@@ -172,9 +178,9 @@ static void auto_attack_with_unit(struct player *pplayer, struct city *pcity,
     set_unit_activity(punit, ACTIVITY_GOTO);
     punit->goto_dest_x=pcity->x;
     punit->goto_dest_y=pcity->y;
-    send_unit_info(0,punit);
+    send_unit_info(NULL, punit);
     
-    do_unit_goto(punit, GOTO_MOVE_ANY, 0);
+    (void) do_unit_goto(punit, GOTO_MOVE_ANY, FALSE);
     
     if (unit_list_find(&map_get_tile(pcity->x, pcity->y)->units, id)) {
       handle_unit_activity_request(punit, ACTIVITY_IDLE);
@@ -208,8 +214,8 @@ static void auto_attack_player(struct player *pplayer)
 
   city_list_iterate(pplayer->cities, pcity) {
     /* fasten things up -- fisch */
-    if ((pcity->city_options & CITYOPT_AUTOATTACK_BITS)
-	&& !pcity->anarchy) {
+    if ((pcity->city_options & CITYOPT_AUTOATTACK_BITS) != 0
+	&& pcity->anarchy == 0) {
       auto_attack_city(pplayer, pcity);
     }
   }
@@ -225,8 +231,8 @@ static void auto_attack_player(struct player *pplayer)
     if(punit->ai.control
        && is_military_unit(punit)
        && punit->activity == ACTIVITY_GOTO
-       && punit->moves_left == get_unit_type(punit->type)->move_rate) {
-      do_unit_goto(punit, GOTO_MOVE_ANY, 0);
+       && punit->moves_left == unit_type(punit)->move_rate) {
+      (void) do_unit_goto(punit, GOTO_MOVE_ANY, FALSE);
     }
   }
   unit_list_iterate_end;
