@@ -25,6 +25,7 @@
 #include "packets.h"
 #include "player.h"
 #include "unit.h"
+#include "events.h"
 
 #include "citytools.h"
 #include "cityturn.h"
@@ -38,38 +39,38 @@
 #include "diplhand.h"
 
 static struct genlist treaties;
-static int did_init_treaties;
+static bool did_init_treaties;
 
 
 /**************************************************************************
 Checks if the player numbers in the packet are sane.
 **************************************************************************/
-static int check_packet(struct packet_diplomacy_info *packet, int check_other)
+static bool check_packet(struct packet_diplomacy_info *packet, bool check_other)
 {
   if (packet->plrno0 >= game.nplayers) {
     freelog(LOG_ERROR, "plrno0 is %d >= game.nplayers (%d).",
 	    packet->plrno0, game.nplayers);
-    return 0;
+    return FALSE;
   }
   if (packet->plrno1 >= game.nplayers) {
     freelog(LOG_ERROR, "plrno1 is %d >= game.nplayers (%d).",
 	    packet->plrno1, game.nplayers);
-    return 0;
+    return FALSE;
   }
   if (check_other && packet->plrno_from >= game.nplayers) {
     freelog(LOG_ERROR, "plrno_from is %d >= game.nplayers (%d).",
 	    packet->plrno_from, game.nplayers);
-    return 0;
-
-    if (packet->plrno_from != packet->plrno0
-	&& packet->plrno_from != packet->plrno1) {
-      freelog(LOG_ERROR, "plrno_from(%d) != plrno0(%d) and plrno1(%d)",
-	      packet->plrno_from, packet->plrno0, packet->plrno1);
-      return 0;
-    }
+    return FALSE;
   }
 
-  return 1;
+  if (packet->plrno_from != packet->plrno0
+      && packet->plrno_from != packet->plrno1) {
+    freelog(LOG_ERROR, "plrno_from(%d) != plrno0(%d) and plrno1(%d)",
+	    packet->plrno_from, packet->plrno0, packet->plrno1);
+    return FALSE;
+  }
+
+  return TRUE;
 }
 
 /**************************************************************************
@@ -81,7 +82,7 @@ struct Treaty *find_treaty(struct player *plr0, struct player *plr1)
   
   if(!did_init_treaties) {
     genlist_init(&treaties);
-    did_init_treaties=1;
+    did_init_treaties = TRUE;
   }
 
   genlist_iterator_init(&myiter, &treaties, 0);
@@ -92,7 +93,7 @@ struct Treaty *find_treaty(struct player *plr0, struct player *plr1)
        (ptreaty->plr0==plr1 && ptreaty->plr1==plr0))
       return ptreaty;
   }
-  return 0;
+  return NULL;
 }
 
 /**************************************************************************
@@ -105,9 +106,9 @@ void handle_diplomacy_accept_treaty(struct player *pplayer,
 {
   struct Treaty *ptreaty;
   struct player *plr0, *plr1, *pgiver, *pdest, *other;
-  int *giver_accept;
+  bool *giver_accept;
 
-  if (!check_packet(packet, 1))
+  if (!check_packet(packet, TRUE))
     return;
 
   plr0 = &game.players[packet->plrno0];
@@ -131,12 +132,7 @@ void handle_diplomacy_accept_treaty(struct player *pplayer,
 
     /* Check that player who accepts can keep what (s)he promises. */
 
-    struct genlist_iterator myiter;
-
-    genlist_iterator_init(&myiter, &ptreaty->clauses, 0);
-
-    for (;ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter)) {
-      struct Clause *pclause = (struct Clause *)ITERATOR_PTR(myiter);
+    clause_list_iterate(ptreaty->clauses, pclause) {
       struct city *pcity;
 
       if (pclause->from == pplayer) {
@@ -168,7 +164,7 @@ void handle_diplomacy_accept_treaty(struct player *pplayer,
 			  pcity->name);
 	    return;
 	  }
-	  if (pcity->improvements[B_PALACE]) {
+	  if (city_got_building(pcity,B_PALACE)) {
 	    notify_player(pplayer,
 			  _("Game: Your capital (%s) is requested, "
 			    "you can't accept treaty."),
@@ -179,7 +175,7 @@ void handle_diplomacy_accept_treaty(struct player *pplayer,
 	case CLAUSE_GOLD:
 	  if (pplayer->economic.gold < pclause->value) {
 	    notify_player(pplayer,
-			  _("Game: You don't have enought gold, "
+			  _("Game: You don't have enough gold, "
 			    "you can't accept treaty."));
 	    return;
 	  }
@@ -188,7 +184,7 @@ void handle_diplomacy_accept_treaty(struct player *pplayer,
 	  ; /* nothing */
 	}
       }
-    }
+    } clause_list_iterate_end;
   }
 
   *giver_accept = ! *giver_accept;
@@ -201,7 +197,7 @@ void handle_diplomacy_accept_treaty(struct player *pplayer,
 			      packet);
 
   if (ptreaty->accept0 && ptreaty->accept1) {
-    struct genlist_iterator myiter;
+    int nclauses = clause_list_size(&ptreaty->clauses);
 
     lsend_packet_diplomacy_info(&plr0->connections,
 				PACKET_DIPLOMACY_CANCEL_MEETING, 
@@ -211,21 +207,23 @@ void handle_diplomacy_accept_treaty(struct player *pplayer,
 				packet);
 
     notify_player(plr0,
-		  _("Game: A treaty containing %d clauses was agreed upon."),
-		  genlist_size(&ptreaty->clauses));
+		  PL_("Game: A treaty containing %d clause was agreed upon.",
+		      "Game: A treaty containing %d clauses was agreed upon.",
+		      nclauses),
+		  nclauses);
     notify_player(plr1,
-		  _("Game: A treaty containing %d clauses was agreed upon."),
-		  genlist_size(&ptreaty->clauses));
-    gamelog(GAMELOG_TREATY, "%s and %s agree to a treaty",
+		  PL_("Game: A treaty containing %d clause was agreed upon.",
+		      "Game: A treaty containing %d clauses was agreed upon.",
+		      nclauses),
+		  nclauses);
+    gamelog(GAMELOG_TREATY, _("%s and %s agree to a treaty"),
 	    get_nation_name_plural(plr0->nation),
 	    get_nation_name_plural(plr1->nation));
 
     /* Check that one who accepted treaty earlier still have everything
        (s)he promised to give. */
 
-    genlist_iterator_init(&myiter, &ptreaty->clauses, 0);
-    for(;ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter)) {
-      struct Clause *pclause=(struct Clause *)ITERATOR_PTR(myiter);
+    clause_list_iterate(ptreaty->clauses, pclause) {
       struct city *pcity;
       if (pclause->from == other) {
 	switch (pclause->type) {
@@ -255,6 +253,13 @@ void handle_diplomacy_accept_treaty(struct player *pplayer,
 			  pcity->name);
 	    goto cleanup;
 	  }
+	  if (city_got_building(pcity,B_PALACE)) {
+	    notify_player(other,
+			  _("Game: Your capital (%s) is requested, "
+			    "you can't accept treaty."),
+			  pcity->name);
+	    goto cleanup;
+	  }
 	  break;
 	case CLAUSE_GOLD:
 	  if (other->economic.gold < pclause->value) {
@@ -273,35 +278,38 @@ void handle_diplomacy_accept_treaty(struct player *pplayer,
 	  ; /* nothing */
 	}
       }
-    }
-
-    genlist_iterator_init(&myiter, &ptreaty->clauses, 0);
-
-    for(;ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter)) {
-      struct Clause *pclause=(struct Clause *)ITERATOR_PTR(myiter);
+    } clause_list_iterate_end;
 
+    clause_list_iterate(ptreaty->clauses, pclause) {
       pgiver = pclause->from;
       pdest = (plr0==pgiver) ? plr1 : plr0;
 
       switch (pclause->type) {
       case CLAUSE_ADVANCE:
-	notify_player(pdest, _("Game: You are taught the knowledge of %s."),
-		      advances[pclause->value].name);
+        /* It is possible that two players open the diplomacy dialog
+         * and try to give us the same tech at the same time. This
+         * should be handled discreetly instead of giving a core dump. */
+        if (get_invention(pdest, pclause->value) == TECH_KNOWN) {
+          break;
+        }
+	notify_player_ex(pdest, -1, -1, E_TECH_GAIN,
+			 _("Game: You are taught the knowledge of %s."),
+			 get_tech_name(pdest, pclause->value));
 
-	notify_embassies(pdest,pgiver,
-			 _("Game: The %s have aquired %s from the %s."),
+	notify_embassies(pdest, pgiver,
+			 _("Game: The %s have acquired %s from the %s."),
 			 get_nation_name_plural(pdest->nation),
-			 advances[pclause->value].name,
+			 get_tech_name(pdest, pclause->value),
 			 get_nation_name_plural(pgiver->nation));
 
-	gamelog(GAMELOG_TECH, "%s acquire %s (Treaty) from %s",
+	gamelog(GAMELOG_TECH, _("%s acquire %s (Treaty) from %s"),
 		get_nation_name_plural(pdest->nation),
-		advances[pclause->value].name,
+		get_tech_name(pdest, pclause->value),
 		get_nation_name_plural(pgiver->nation));
 
 	do_dipl_cost(pdest);
 
-	found_new_tech(pdest, pclause->value, 0, 1);
+	found_new_tech(pdest, pclause->value, FALSE, TRUE);
 	break;
       case CLAUSE_GOLD:
 	notify_player(pdest, _("Game: You get %d gold."), pclause->value);
@@ -321,7 +329,6 @@ void handle_diplomacy_accept_treaty(struct player *pplayer,
       case CLAUSE_CITY:
 	{
 	  struct city *pcity = find_city_by_id(pclause->value);
-	  struct city *pnewcity = NULL;
 
 	  if (!pcity) {
 	    freelog(LOG_NORMAL,
@@ -330,17 +337,15 @@ void handle_diplomacy_accept_treaty(struct player *pplayer,
 	    break;
 	  }
 
-	  notify_player(pdest, _("Game: You receive city of %s from %s."),
-			pcity->name, pgiver->name);
+	  notify_player_ex(pdest, pcity->x, pcity->y, E_CITY_TRANSFER,
+			   _("Game: You receive city of %s from %s."),
+			   pcity->name, pgiver->name);
 
-	  notify_player(pgiver, _("Game: You give city of %s to %s."),
-			pcity->name, pdest->name);
+	  notify_player_ex(pgiver, pcity->x, pcity->y, E_CITY_LOST,
+			   _("Game: You give city of %s to %s."),
+			   pcity->name, pdest->name);
 
-	  pnewcity = transfer_city(pdest, pcity, -1, 1, 1, 0);
-	  if (!pnewcity) {
-	    freelog(LOG_NORMAL, "Transfer city returned no city - skipping clause.");
-	    break;
-	  }
+	  transfer_city(pdest, pcity, -1, TRUE, TRUE, FALSE);
 	  break;
 	}
       case CLAUSE_CEASEFIRE:
@@ -348,48 +353,56 @@ void handle_diplomacy_accept_treaty(struct player *pplayer,
 	pgiver->diplstates[pdest->player_no].turns_left=16;
 	pdest->diplstates[pgiver->player_no].type=DS_CEASEFIRE;
 	pdest->diplstates[pgiver->player_no].turns_left=16;
-	notify_player(pgiver, _("Game: You agree on a cease-fire with %s."),
-		      pdest->name);
-	notify_player(pdest, _("Game: You agree on a cease-fire with %s."),
-		      pgiver->name);
+	notify_player_ex(pgiver, -1, -1, E_TREATY_CEASEFIRE,
+			 _("Game: You agree on a cease-fire with %s."),
+			 pdest->name);
+	notify_player_ex(pdest, -1, -1, E_TREATY_CEASEFIRE,
+			 _("Game: You agree on a cease-fire with %s."),
+			 pgiver->name);
 	check_city_workers(plr0);
 	check_city_workers(plr1);
 	break;
       case CLAUSE_PEACE:
 	pgiver->diplstates[pdest->player_no].type=DS_PEACE;
 	pdest->diplstates[pgiver->player_no].type=DS_PEACE;
-	notify_player(pgiver, _("Game: You agree on peace with %s."),
-		      pdest->name);
-	notify_player(pdest, _("Game: You agree on peace with %s."),
-		      pgiver->name);    
+	notify_player_ex(pgiver, -1, -1, E_TREATY_PEACE,
+			 _("Game: You agree on a peace treaty with %s."),
+			 pdest->name);
+	notify_player_ex(pdest, -1, -1, E_TREATY_PEACE,
+			 _("Game: You agree on a peace treaty with %s."),
+			 pgiver->name);
 	check_city_workers(plr0);
 	check_city_workers(plr1);
 	break;
       case CLAUSE_ALLIANCE:
 	pgiver->diplstates[pdest->player_no].type=DS_ALLIANCE;
 	pdest->diplstates[pgiver->player_no].type=DS_ALLIANCE;
-	notify_player(pgiver, _("Game: You agree on an alliance with %s."),
-		      pdest->name);
-	notify_player(pdest, _("Game: You agree on an alliance with %s."),
-		      pgiver->name);
+	notify_player_ex(pgiver, -1, -1, E_TREATY_ALLIANCE,
+			 _("Game: You agree on an alliance with %s."),
+			 pdest->name);
+	notify_player_ex(pdest, -1, -1, E_TREATY_ALLIANCE,
+			 _("Game: You agree on an alliance with %s."),
+			 pgiver->name);
 	check_city_workers(plr0);
 	check_city_workers(plr1);
 	break;          
       case CLAUSE_VISION:
 	give_shared_vision(pgiver, pdest);
-	notify_player(pgiver, _("Game: You give shared vision to %s."),
-		      pdest->name);
-	notify_player(pdest, _("Game: %s gives you shared vision."),
-		      pgiver->name);
+	notify_player_ex(pgiver, -1, -1, E_TREATY_SHARED_VISION,
+			 _("Game: You give shared vision to %s."),
+			 pdest->name);
+	notify_player_ex(pdest, -1, -1, E_TREATY_SHARED_VISION,
+			 _("Game: %s gives you shared vision."),
+			 pgiver->name);
 	break;
       }
 
-    }
+    } clause_list_iterate_end;
   cleanup:      
     genlist_unlink(&treaties, ptreaty);
     free(ptreaty);
-    send_player_info(plr0, 0);
-    send_player_info(plr1, 0);
+    send_player_info(plr0, NULL);
+    send_player_info(plr1, NULL);
   }
 }
 
@@ -403,7 +416,7 @@ void handle_diplomacy_remove_clause(struct player *pplayer,
   struct Treaty *ptreaty;
   struct player *plr0, *plr1, *pgiver;
 
-  if (!check_packet(packet, 1))
+  if (!check_packet(packet, TRUE))
     return;
 
   plr0=&game.players[packet->plrno0];
@@ -430,7 +443,7 @@ void handle_diplomacy_create_clause(struct player *pplayer,
   struct Treaty *ptreaty;
   struct player *plr0, *plr1, *pgiver;
 
-  if (!check_packet(packet, 1))
+  if (!check_packet(packet, TRUE))
     return;
 
   plr0=&game.players[packet->plrno0];
@@ -449,7 +462,7 @@ void handle_diplomacy_create_clause(struct player *pplayer,
        */
       if (packet->clause_type == CLAUSE_CITY){
 	struct city *pcity = find_city_by_id(packet->value);
-	if (pcity && !map_get_known_and_seen(pcity->x, pcity->y, plr1->player_no))
+	if (pcity && !map_get_known_and_seen(pcity->x, pcity->y, plr1))
 	  give_citymap_from_player_to_player(pcity, plr0, plr1);
       }
 
@@ -501,7 +514,7 @@ void handle_diplomacy_cancel_meeting(struct player *pplayer,
 {
   struct player *plr0, *plr1, *theother;
 
-  if (!check_packet(packet, 0))
+  if (!check_packet(packet, FALSE))
     return;
 
   plr0=&game.players[packet->plrno0];
@@ -521,7 +534,7 @@ void handle_diplomacy_init(struct player *pplayer,
   struct packet_diplomacy_info pa;
   struct player *plr0, *plr1;
 
-  if (!check_packet(packet, 0))
+  if (!check_packet(packet, FALSE))
     return;
 
   plr0=&game.players[packet->plrno0];
@@ -569,12 +582,11 @@ void send_diplomatic_meetings(struct connection *dest)
   struct Treaty *ptreaty;
   struct player *pplayer = dest->player;
 
-  if (pplayer==NULL) {
+  if (!pplayer) {
     return;
   }
   players_iterate(other_player) {
     if ( (ptreaty=find_treaty(pplayer, other_player))) {
-      struct genlist_iterator myiter;
       struct packet_diplomacy_info packet;
       
       packet.plrno0 = pplayer->player_no;
@@ -582,18 +594,14 @@ void send_diplomatic_meetings(struct connection *dest)
       
       send_packet_diplomacy_info(dest, PACKET_DIPLOMACY_INIT_MEETING, &packet);
       
-      genlist_iterator_init(&myiter, &ptreaty->clauses, 0);
-      
-      for(;ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter)) {
-	struct Clause *pclause=(struct Clause *)ITERATOR_PTR(myiter);
-
+      clause_list_iterate(ptreaty->clauses, pclause) {
 	packet.clause_type = pclause->type;
 	packet.plrno_from = pclause->from->player_no;
 	packet.value = pclause->value;
   
 	send_packet_diplomacy_info(dest, PACKET_DIPLOMACY_CREATE_CLAUSE,
 				   &packet);
-      }
+      } clause_list_iterate_end;
     }
   } players_iterate_end;
 }
